#! /usr/bin/perl
##
##
## This module is part of ML and provides some tools

package tools;

use Mail::Internet;
use Mail::Header;
use Conf;
use Language;
use Log;

## RCS identification.
#my $id = '@(#)$Id: tools.pl,v 1.3 1998/12/22 13:52:43 sympa Exp $';

## Sorts the list of adresses by domain name
## Input : users hash
## Sort by domain.
sub sortbydomain {
   my($x, $y) = @_;
   $x = join('.', reverse(split(/[@\.]/, $x)));
   $y = join('.', reverse(split(/[@\.]/, $y)));
   #print "$x $y\n";
   $x cmp $y;
}

## Safefork does several tries before it gives up.
## Do 3 trials and wait 10 seconds between each.
## Exit with a fatal error is fork failed after all
## tests have been exhausted.
sub safefork {
   my($i, $pid);
   
   for ($i = 1; $i < 360; $i++) {
      my($pid) = fork;
      return $pid if (defined($pid));
      do_log ('warning', "Can't create new process in safefork: %m");
      ## should send a mail to the listmaster
      sleep(10 * $i);
   }
   fatal_err("Can't create new process in safefork: %m");
   ## No return.
}

@avoid_hdr = (
        'help',
        'ind(ex)?',
        'lists?',
        '(please\s+)?(add|unsub?(scribe)?|remove|del|sub\s|sub?s?cribe|sign?o?f?f?)',
        'rev(iew)?\s+\S+',
        'stats\s+\S+',
        'get\s+\S+\s+\S+',
        'set\s+\S+\s+(no)?(mail|conceal|digest)',
        'purge\s+\S+\s+\S+',
        'exp(ire)?\s+\S+\s+\S+\s+\S+',
        'exp(ire)?del\s+\S+',
        'exp(ire)?ind(ex)?\s+\S+',
        'ind(ex)?exp(ire)?\s+\S+',
        'mod(eration)?ind(ex)?\s+\S+',
        'ind(ex)?mod(eration)?\s+\S+',
	'rev(iew)?\s+\S+',
        'dis(tribute)?\s+\S+\s+\S+',
        'rej(ect)?\s+\S+\s+\S+',
        '(re)?con(firm)?\s+\S+',
	'rev(iew)?\s+\S+',
);

## Check for commands in the body of the message. Returns true
## if there are some commands in it.
sub checkcommand {
   my($msg, $sender) = @_;
   do_log('debug2', 'tools::checkcommand(%s)', $sender);

   my($avoid, $i);

   return 0 if ($#{$msg->body} >= 15);  ## More than 15 lines in the text.
   my $hdr = $msg->head;

   ## Check for commands in the subject.
   my $subject = $msg->get('Subject');
   if ($subject) {
      foreach $avoid (@avoid_hdr) {
         if ($subject =~ /^\s*(quiet)?($avoid)(\s+|$)/im) {
            &rejectMessage($msg, $sender);
            return 1;
         }
      }
   }
   foreach $i (@{$msg->body}) {
      foreach $avoid (@avoid_hdr) {
         if ($i =~ /^\s*(quiet)?($avoid)(\s+|$)/im) {  ## Suspicious line
            &rejectMessage($msg, $sender);
            return 1;
         }
      }
      ## Control is only applied to first non-blank line
      last unless $i =~ /^\s*$/;
      
   }
   return 0;
}

sub rejectMessage {
   my($msg, $sender) = @_;
   do_log('debug2', 'tools::rejectMessage(%s)', $sender);

   *REJ = smtp::smtpto($Conf{'request'}, \$sender);
   print REJ "To: $sender\n";
   print REJ "Subject: [sympa] " . Msg(5, 2, "Misadressed message ?") . "\n";
   printf REJ "MIME-Version: %s\n", Msg(12, 1, '1.0');
   printf REJ "Content-Type: text/plain; charset=%s\n", Msg(12, 2, 'us-ascii');
   printf REJ "Content-Transfer-Encoding: %s\n", Msg(12, 3, '7bit');
   print REJ "\n";
   printf REJ Msg(5, 3, "\
Your message has been sent to a list but it seems it contains commands like
subscribe, signoff, help, index, get, ...

If your message did really contain a command, please note that such messages
must be sent to %s only.

If it happens that your message was by mistake considered as containing
commands, then please contact the manager of this service %s
so that he can take care of your problem.

Thank you for your attention.

------ Beginning of the suspect message --------
"), "$Conf{'sympa'}", $Conf{'request'};
   $msg->print(\*REJ);
   print REJ Msg(5, 4, "------- Fin message suspect ---------\n");
   close(REJ);
}

## return a hash from the edit_list_conf file
sub load_edit_list_conf {

    my $file;
    my $conf ;
    
    if (-r "$Conf{'etc'}/edit_list.conf") {
	$file = "$Conf{'etc'}/edit_list.conf";
    }elsif (-r "/var/lib/sympa/bin/etc/edit_list.conf") {
	$file = "/var/lib/sympa/bin/etc/edit_list.conf";
    }else {
	&do_log('info','Cannot find edit_list.conf');
	return undef;
    }

    unless (open (FILE, $file)) {
	&do_log('info','Unable to open config file %s', $file);
	return undef;
    }

    while (<FILE>) {
	next if /^\s*(\#.*|\s*)$/;

	if (/^\s*(\S+)\s+(listmaster|privileged_owner|owner|editor|subscriber|any)\s+(read|write|hidden)\s*$/i) {
	    $conf->{$1}{$2} = $3;
	}else{
	    &do_log ('info', 'unknown parameter in %s  (Ignored) %s', "$Conf{'etc'}/edit_list.conf",$_ );
	    next;
	}
    }
    
    close FILE;
    return $conf;
}


## return a hash from the edit_list_conf file
sub load_create_list_conf {

    my $file;
    my $conf ;
    
    if (-r "$Conf{'etc'}/create_list.conf") {
	$file = "$Conf{'etc'}/create_list.conf";
    }elsif (-r "/var/lib/sympa/bin/etc/create_list.conf") {
	$file = "/var/lib/sympa/bin/etc/create_list.conf";
    }else {
	&do_log('info','enable to read /var/lib/sympa/bin/etc/create_list.conf');
	return undef;
    }

    unless (open (FILE, $file)) {
	&do_log('info','Unable to open config file %s', $file);
	return undef;
    }

    while (<FILE>) {
	next if /^\s*(\#.*|\s*)$/;

	if (/^\s*(\S+)\s+(read|hidden)\s*$/i) {
	    $conf->{$1} = $2;
	}else{
	    &do_log ('info', 'unknown parameter in %s  (Ignored) %s', "$Conf{'etc'}/create_list.conf",$_ );
	    next;
	}
    }
    
    close FILE;
    return $conf;
}

## Loads the list of topics
sub load_topics_conf {
    do_log('debug2', 'tools::load_topics_conf');

    my $conf_file = "$Conf{'etc'}/topics.conf";
    my $topics = {};

    unless (-r $conf_file) {
	&do_log('info',"Enable to read $conf_file");
	return undef;
    }
    
    unless (open (FILE, $conf_file)) {
	&do_log('info',"Unable to open config file $conf_file");
	return undef;
    }

    my $index;
    while (<FILE>) {
	if (/^([\w\/]+)\s+(.+)\s*$/) {
	    my @tree = split '/', $1;
	    $index++;
	    
	    if ($#tree == 0) {
		$topics->{$tree[0]}{'title'} = $2;
		$topics->{$tree[0]}{'order'} = $index;
	    }else {
		my $subtopic = join ('/', @tree[1..$#tree]);
		$topics->{$tree[0]}{'sub'}{$subtopic} = &_add_topic($subtopic,$2);
	    }
	}
    }
    close FILE;

    return $topics;
}

sub _add_topic {
    my ($name, $title) = @_;
    my $topic = {};

    my @tree = split '/', $name;
    if ($#tree == 0) {
	return {'title' => $title};
    }else {
	$topic->{'sub'}{$name} = &_add_topic(join ('/', @tree[1..$#tree]), $title);
	return $topic;
    }
}

sub get_list_list_tpl {

    my $list_conf;
    my $list_templates ;
    unless ($list_conf = &load_create_list_conf()) {
	return undef;
    }

    if (opendir(DIR, '/var/lib/sympa/bin/etc/create_list_templates')) {
	foreach my $template ( sort grep (!/^\./,readdir(DIR))) {
	    if ($list_conf->{$template} eq 'read') {
		$list_templates->{$template}{'path'} = '/var/lib/sympa/bin/etc/create_list_templates/';
		next;
	    }
	    next if ($list_conf->{$template} eq 'hidden') ;
	    next if ($list_conf->{'default'} eq 'hidden') ;
            $list_templates->{$template}{'path'} = '/var/lib/sympa/bin/etc/create_list_templates/';
	}
	closedir(DIR);
    }

    if (opendir(DIR, "$Conf{'etc'}/create_list_templates")) {
	foreach my $template (sort grep (!/^\./,readdir(DIR))) {
	    if ($list_conf->{$template} eq 'read') {
		$param->{'templates'}{$template}{'path'} = "$Conf{'etc'}/create_list_templates/" ;
		next;
	    }
	    next if ($list_conf->{$template} eq 'hidden') ;
	    next if ($list_conf->{'default'} eq 'hidden') ;
            $list_templates->{$template}{'path'} = "$Conf{'etc'}/create_list_templates/";   
	}	

	closedir(DIR);
    }
    foreach my $template ($list_templates){
	
    }

    return ($list_templates);
}

sub smime_sign_check {

    my $msgfile = shift;
    my $sender = shift;
    $sender= lc($sender);

    do_log('debug2', 'tools::smime_sign_check (%s,%s)', $msgfile,$sender);

    my $is_signed = {};
    $is_signed->{'body'} = undef;   
    $is_signed->{'subject'} = undef;

    my $verify ;

    ## first step is the msg signing OK ; /tmp/sympa-smime.$$ is created
    ## to store the signer certificat for step two. I known, that's durty.
    do_log('debug2', '%s smime -in %s -verify %s',$Conf{'openssl'},$msgfile,$Conf{'trusted_ca_options'});


    $verify = `$Conf{'openssl'} smime -in $msgfile -verify  $Conf{'trusted_ca_options'} -signer /tmp/smime-sender.$$ 2>&1` ;
    if ($? != 0) {	
	do_log('err', "unable to verify smime signature from $sender $verify");
	return undef ;
    }
    ## second step is the message signer match the sender
    ## a better analyse should be performed to extract the signer email. 
    my $signer = `cat /tmp/smime-sender.$$ | $Conf{'openssl'}  x509 -subject -noout`;

    ## store the user certificat in order to use it for enscryption
    

    if ($signer =~ /email=$sender/i) {
	do_log('debug', "S/MIME signed message, signature checked and sender match signer(%s)",$signer);
        ## store the signer certificat
	unless (-d "$Conf{'home'}/X509-user-certs") {
	    if ( mkdir ("$Conf{'home'}/X509-user-certs", 0775)) {
		do_log('info', "creating spool $Conf{'home'}/X509-user-certs");
	    }else{
		do_log('err', "enable to create user certificat directory $Conf{'home'}/X509-user-certs");
	    }
	}
	my $filename = "$Conf{'home'}/X509-user-certs/".&escape_chars($sender);

	open (CERTIF,"/tmp/smime-sender.$$");
	if (open (USERCERTIF, "> $filename")) {
	    while (<CERTIF>) {
		print USERCERTIF;
	    }
	}else{
	    &do_log('err','Unable to rename /tmp/smime-sender.%s,%s',$$,$filename);
	}
	unlink('/tmp/smime-sender.$$');	

	$is_signed->{'body'} = 'smime';
	    &do_log('debug2',"xxxxxxxxxxxxxxxxxxxxxxxxxx is-signed->body = smime");
	# futur version should check if the subject was part of the SMIME signature.
	$is_signed->{'subject'} = undef;
	return $is_signed;
    }else{
	unlink('/tmp/smime-sender.$$');	
	do_log('notice', "S/MIME signed message, sender($sender) do NOT match signer($signer)",$sender,$signer);
	return undef;
    }
    return undef ;    
}

## Make a multipart/alternative, a singlepart
sub as_singlepart {
    my ($msg, $preferred_type) = @_;
    my $done = 0;
    
    my @parts = $msg->parts();
    foreach my $index (0..$#parts) {
	if ($parts[$index]->effective_type() eq $preferred_type) {
	    ## Only keep the first matching part
	    $msg->parts([$parts[$index]]);
	    $msg->make_singlepart();
	    $done = 1;
	    last;
	}
    }

    return $done;
}


## Escape weird characters
sub escape_chars {
    my $s = shift;

    $s =~ s/\%/\%25/g;
    $s =~ s/\"/\%22/g;
    $s =~ s/\s/\%20/g;
    $s =~ s/\xa5/\%a5/g;
    $s =~ s/\//\%a5/g; ## Special traetment for '/'
    $s =~ s/\:/\%3a/g;
    
    return $s;
}

## Unescape weird characters
sub unescape_chars {
    my $s = shift;

    $s =~ s/\%25/\%/g;
    $s =~ s/\%22/\"/g;
    $s =~ s/\%20/ /g;
    $s =~ s/\%a5/\//g;  ## Special traetment for '/'
    $s =~ s/\%3a/\:/g;
    
    return $s;
}


1;



