#!/usr/bin/perl

## Copyright 1999 Comit Rseaux des Universits
## web interface to Sympa mailing lists manager
## Sympa: http://listes.cru.fr/sympa/

## Authors :
##           Serge Aumont <sa@cru.fr>
##           Olivier Salan <os@cru.fr>

## Change this to point to your Sympa bin directory
use lib '/var/lib/sympa/bin';
use lib '/var/lib/sympa/bin';

## Template parser
require "/var/lib/sympa/bin/parser.pl";

## Sympa API
use List;
use smtp;
use Conf;
use Commands;
use Log;
use Getopt::Std;

use Mail::Header;
use Mail::Address;

require "/var/lib/sympa/bin/mail.pl";
require "/var/lib/sympa/bin/msg.pl";
require "/var/lib/sympa/bin/tools.pl";

## WWSympa librairies
require "/var/lib/sympa/bin/wws-lib.pl";
require "/var/lib/sympa/bin/cookie-lib.pl";

## Configuration
$wwsconf = {};

## Change to your wwsympa.conf location
$conf_file = '/etc/wwsympa.conf';
$sympa_conf_file = '/etc/sympa.conf';

## Load config 
unless ($wwsconf = &load_config($conf_file)) {
    &message('unable to load config file');
    return undef;
}

## Load sympa config
unless (&Conf::load( $sympa_conf_file )) {
    &message('config_error');
    &do_log('info','unable to load sympa config file');
    exit (-1);
}

$mime_types = &load_mime_types();

if ($wwsconf->{'use_fast_cgi'}) {
    require CGI::Fast;
}else {
    require CGI;
}


$list;
$param = {};
$loop = 0;

## subroutines
%comm = ('home' => 'do_home',
	 'logout' => 'do_logout',
	 'loginrequest' => 'do_loginrequest',
	 'login' => 'do_login',
	 'subscribe' => 'do_subscribe',
	 'subrequest' => 'do_subrequest',
	 'signoff' => 'do_signoff',
	 'sigrequest' => 'do_sigrequest',
	 'which' => 'do_which',
	 'lists' => 'do_lists',
	 'info' => 'do_info',
	 'review' => 'do_review',
	 'search' => 'do_search',
	 'pref', => 'do_pref',
	 'setpref' => 'do_setpref',
	 'setpasswd' => 'do_setpasswd',
	 'remindpasswd' => 'do_remindpasswd',
	 'sendpasswd' => 'do_sendpasswd',
	 'choosepasswd' => 'do_choosepasswd',	
	 'viewfile' => 'do_viewfile',
	 'set' => 'do_set',
	 'admin' => 'do_admin',
	 'add_request' => 'do_add_request',
	 'add' => 'do_add',
	 'del' => 'do_del',
	 'modindex' => 'do_modindex',
	 'reject' => 'do_reject',
	 'reject_notify' => 'do_reject_notify',
	 'distribute' => 'do_distribute',
	 'viewmod' => 'do_viewmod',
	 'editfile' => 'do_editfile',
	 'savefile' => 'do_savefile',
	 'arc' => 'do_arc',
	 'remove_arc' => 'do_remove_arc',
	 'arcsearch_form' => 'do_arcsearch_form',
	 'arcsearch' => 'do_arcsearch',
	 'rebuildarc' => 'do_rebuildarc',
	 'rebuildallarc' => 'do_rebuildallarc',
	 'serveradmin' => 'do_serveradmin',
	 'help' => 'do_help',
	 'edit_list_request' => 'do_edit_list_request',
	 'edit_list' => 'do_edit_list',
	 'create_list_request' => 'do_create_list_request',
	 'create_list' => 'do_create_list',
	 'get_pending_lists' => 'do_get_pending_lists', 
	 'set_pending_list_request' => 'do_set_pending_list_request', 
	 'install_pending_list' => 'do_install_pending_list', 
	 'submit_list' => 'do_submit_list',
	 'editsubscriber' => 'do_editsubscriber',
	 'viewbounce' => 'do_viewbounce',
	 'viewconfig' => 'do_viewconfig',
 	 'reviewbouncing' => 'do_reviewbouncing',
 	 'resetbounce' => 'do_resetbounce',
 	 'scenario_test' => 'do_scenario_test',
	 'search_list' => 'do_search_list',
	 'shared' => 'do_shared',
	 'upload_request' => 'do_upload_request',
	 'upload' => 'do_upload'
	 );

## Arguments awaited in the PATH_INFO, depending on the action 
%action_args = ('editfile' => ['list','file'],
		'viewfile' => ['list','file'],
		'sendpasswd' => ['email'],
		'choosepasswd' => ['email','passwd'],
		'lists' => ['topic','subtopic'],
		'login' => ['email','passwd','previous_action','previous_list'],
		'loginrequest' => ['previous_action','previous_list'],
		'logout' => ['previous_action','previous_list'],
		'remindpasswd' => ['previous_action','previous_list'],
		'reject' => ['list','id'],
		'distribute' => ['list','id'],
		'modindex' => ['list'],
		'viewmod' => ['list','id','file'],
		'viewfile' => ['list','file'],
		'add' => ['list','email'],
		'add_request' => ['list'],
		'del' => ['list','email'],
		'editsubscriber' => ['list','email','previous_action'],
		'viewbounce' => ['list','email'],
		'viewconfig' => ['list','email'],
		'resetbounce' => ['list','email'],
		'review' => ['list','page','size','sortby'],
		'reviewbouncing' => ['list','page','size'],
		'arc' => ['list','month','arc_file'],
		'arcsearch_form' => ['list','archive_name'],
		'rebuildarc' => ['list','month'],
		'rebuildallarc' => [],
		'home' => [],
		'help' => ['help_topic'],
		'subscribe' => ['list','email','passwd'],
		'subrequest' => ['list','email'],
		'signoff' => ['list','email','passwd'],
		'sigrequest' => ['list','email'],
		'set' => ['list','email','reception','gecos'],
		'serveradmin' => [],
		'get_pending_lists' => [],
                'set_pending_list_request' => ['list'],
                'install_pending_list' => ['list'],
		'default' => ['list'],
		'search_list' => ['filter'],
		'shared' => ['list','@path']
		);

## subtitles
%title = ('home' => {'fr' => 'Accueil',
		     'es' => 'Principal',
		     'us' => 'Home'},
	  'logout' => {'fr' => 'Logout',
		       'es' => 'Logout',
		       'us' => 'Logout'},
	  'loginrequest' => {'fr' => 'Login',
			     'es' => 'Login',
			     'us' => 'Login'},
	  'login' => {'fr' => 'Login',
		      'es' => 'Login',
		      'us' => 'Login'},
	  'subscribe' => {'fr' => 'Abonnement',
			  'es' => 'Subscripcin',
			  'us' => 'Subscription'},
	  'subrequest' => {'fr' => 'Demande d\'abonnement',
			   'es' => 'Solicitud de Subscripcin',
			   'us' => 'Subscription request'},
	  'signoff' => {'fr' => 'Dsabonnement',
			'es' => 'Anulacin subscripcin',
			'us' => 'Unsubscription'},
	  'sigrequest' => {'fr' => 'Demande de dsabonnement',
			   'es' => 'Anulacin Subscripcin',
			   'us' => 'Unsubscription request'},
	  'which' => {'fr' => 'Vos abonnements',
		      'es' => 'Sus subscripciones',
		      'us' => 'Your subscriptions'},
	  'lists' => {'fr' => 'Les listes publiques',
		      'es' => 'Listas pblicas',
		      'us' => 'Public lists'},
	  'info' => {'fr' => 'Info',
		     'es' => 'Info',
		     'us' => 'Info'},
	  'review' => {'fr' => 'La liste des abonns',
		       'es' => 'Lista de subscriptores',
		       'us' => 'Subscribers list'},
	  'search' => {'fr' => 'Recherche parmi les abonns',
		       'es' => 'Buscar subscriptores',
		       'us' => 'Search subscribers'},
	  'pref' => {'fr' => 'Prfrences',
		     'es' => 'Preferencias',
		     'us' => 'Preferences'},
	  'setpref' => {'fr' => 'Mise  jour des prfrences utilisateur',
			'es' => 'Preferencias de usuario actualizadas',
			'us' => 'User preferences update'},
	  'setpasswd' => {'fr' => 'Changement de mot de passe',
			  'es' => 'Cambio de contrasea',
			  'us' => 'Password update'},
	  'remindpasswd' => {'fr' => 'Rappel du mot de passe',
			     'es' => 'Recordar la contrasea',
			     'us' => 'Remind of the password'},
	  'sendpasswd' => {'fr' => 'Choix du mot de passe',
			   'es' => 'Enviar la contrasea',
			   'us' => 'Choosing your password'},
	  'choosepasswd' => {'fr' => 'Chosissez un mot de passe',
			     'es' => 'Elegir una contrasea',
			   'us' => 'Choose a password'},
	  'set' => {'fr' => 'Option d\'abonnement',
		    'es' => 'Opciones de subscripcin',
		    'us' => 'Subscription option'},
	  'admin' => {'fr' => 'Admin',
		      'es' => 'Admin',
		      'us' => 'Admin'},
	  'add_request' => {'fr' => 'Ajout d\'abonns',
			    'es' => 'Aadir unos subscriptores',
			    'us' => 'Add subscribers'},
	  'add' => {'fr' => 'Ajout d\'un abonn',
		    'es' => 'Aadir un subscriptor',
		    'us' => 'Add a subscriber'},
	  'del' => {'fr' => 'Suppression d\'un abonn',
		    'es' => 'Borrar un subscriptor',
		    'us' => 'Delete a subscriber'},
	  'modindex' => {'fr' => 'Modration',
			 'es' => 'Moderacin',
			 'us' => 'Moderating'},
	  'reject' => {'fr' => 'Rejeter le message',
		       'es' => 'Rechazar el mensaje',
		       'us' => 'Reject message'},
	  'distribute' => {'fr' => 'Distribuer le message',
			   'es' => 'Distribuir el mensaje',
			   'us' => 'Distribute message'},
	  'viewmod' => {'fr' => 'Voir le message',
			'es' => 'Ver el mensaje',
			'us' => 'View message'},
	  'editfile' => {'fr' => 'Editer le fichier %s',
			 'es' => 'Editar el fichero %s',
			 'us' => 'Edit file %s'},
	  'viewfile' => {'fr' => '%s',
			 'es' => '%s',
			 'us' => '%s'},
	  'savefile' => {'fr' => 'Sauver le fichier %s',
			 'es' => 'Guardar el fichero %s',
			 'us' => 'Save file %s'},
	  'arc' => {'fr' => 'Archives',
		    'es' => 'Archivos',
		    'us' => 'Archives'},
	  'arcsearch_form' => {'fr' => 'Recherche dans les archives',
			       'us' => 'Search in the archives'},
	  'arcsearch' => {'fr' => 'Rsultats des recherches dans les archives',
			  'us' => 'Search results in the archives'},
	  'rebuildarc' => {'fr' => 'Reconstruction des archives',
			   'es' => 'Reconstruir los archivos',
			   'us' => 'Rebuild archives'},
	  'rebuildallarc' => {'fr' => 'Reconstruction de toutes les archives',
			   'es' => 'Reconstruir todos los archivos',
			   'us' => 'Rebuild all archives'},
	  'serveradmin' => {'fr' => 'Admin serveur',
			    'es' => 'Admin Servidor',
			    'us' => 'Server admin'},
	  'help' => {'fr' => 'Aide',
		     'es' => 'Ayuda',
		     'us' => 'Help'},
	  'edit_list_request' => {'fr' => 'Edition de la liste',
				  'es' => 'Edicin de la lista',
				  'us' => 'Edit list'},
	  'edit_list' => {'fr' => 'Edition de la liste',
			  'es' => 'Edicin de la lista',
			  'us' => 'Edit list'},
	  'create_list_request' => {'fr' => 'Cration de liste',
				    'es' => 'Creacin de lista',
				    'us' => 'Create a list'},
	  'create_list' => {'fr' => 'Cration de liste',
			   'es' => 'Creacin de lista',
			   'us' => 'Create a list'},
	  'set_pending_list_request' => {'fr' => 'Set list status',
			   'es' => 'Set list status',
			   'us' => 'Set list status'},
	  'get_pending_lists' => {'fr' => 'La liste des listes en attente',
			   'es' => 'List of pending lists',
			   'us' => 'List of pending lists'},
	  'install_pending_list' => {'fr' => 'Install a list',
			   'es' => 'Install a list',
			   'us' => 'Install a list'},
	  'submit_list' => {'fr' => 'Mise  jour de la liste',
			    'es' => 'Actualizar la lista',
			    'us' => 'Update list'},
 	  'editsubscriber' => {'fr' => 'Edition d\'un abonn',
			       'es' => 'Modificar un subscriptor',
			       'us' => 'Subscriber edition'},
 	  'viewbounce' => {'fr' => 'Le dernier rapport de non-remise',
			   'es' => 'El ltimo error al enviar',
			   'us' => 'Last bounce'},
 	  'viewconfig' => {'fr' => 'Le fichier de config',
			   'es' => 'List config file',
			   'us' => 'List config file'},
	  'reviewbouncing' => {'fr' => 'Les adresses en erreur',
			       'es' => 'Las direcciones con errores',
			       'us' => 'Bouncing addresses'},
	  'resetbounce' => {'fr' => 'Effacer les erreurs',
			    'es' => 'Inicializar los errores de envo',
			    'us' => 'Reset bounces'},
	  'scenario_test' => {'fr' => 'test en situation des scnarios',
			      'es' => 'Comprobando escenario instalado',
			      'us' => 'testing installed scenari'},
	  'search_list' => {'fr' => 'Recherche parmi les listes',
			    'es' => 'Buscar listas',
			    'us' => 'Search lists'},
	  'shared' => {'fr' => 'Partage de documents',
		       'es' => 'Shared documents',
		       'us' => 'Shared documents'},
	  );

## Open log

$wwsconf->{'log_facility'}||= $Conf{'syslog'};

&Log::do_openlog($wwsconf->{'log_facility'}, $Conf{'log_socket_type'}, 'wwsympa');
&do_log('info', 'WWSympa started');

## Set locale configuration
Language::SetLang($Conf{'lang'});

unless ($List::use_db = &List::probe_db()) {
    &message('no_database');
    &do_log('info','WWSympa requires a RDBMS to run');
}

my $pinfo = &List::_apply_defaults();

my (%in, $query);

## Main loop
while ($query = &new_loop()) {

    ## Get params in a hash
    %in = $query->Vars;

    ## Free terminated sendmail processes
    &smtp::reaper;

    ## Parse CGI parameters
#    &CGI::ReadParse();

    ## Get PATH_INFO parameters
    &get_parameters();

    ## Sympa parameters in $param->{'conf'}
    $param->{'conf'} = \%Conf;

    $param->{'path_cgi'} = $ENV{'SCRIPT_NAME'};
    $param->{'version'} = $Version::Version;
    $param->{'date'} = &POSIX::strftime("%d %b %Y at %H:%M:%S", localtime(time));

    ## Change to list root
    unless (chdir($Conf{'home'})) {
	&message('chdir_error');
	&wwslog('info','unable to change directory');
	exit (-1);
    }

    ## Authentication    
    if ($ENV{'HTTP_COOKIE'}) {
	$param->{'user'}{'email'} = &get_email_from_cookie($Conf{'cookie'});

	if (&List::is_user_db($param->{'user'}{'email'})) {
	    $param->{'user'} = &List::get_user_db($param->{'user'}{'email'});
	}
	
	## For the parser to display an empty field instead of [xxx]
	$param->{'user'}{'gecos'} ||= '';
	$param->{'user'}{'cookie_delay'} ||= $wwsconf->{'cookie_expire'};
	$param->{'user'}{'init_passwd'} = 1 if ($param->{'user'}{'password'} =~ /^INIT/);
    }
    
    ## Action
    my $action = $in{'action'} || 'home';
    $param->{'lang'} = $param->{'user'}{'lang'} || $Conf{'lang'};
    $param->{'remote_addr'} = $ENV{'REMOTE_ADDR'} ;
    $param->{'remote_host'} = $ENV{'REMOTE_HOST'};

    ## Session loop
    while ($action) {

	unless (&check_param()) {
	    &message('wrong_param');
	    &wwslog('info','Wrong parameters');
	    last;
	}

	$param->{'host'} = $list->{'admin'}{'host'} || $Conf{'host'};
	
	## language ( $ENV{'HTTP_ACCEPT_LANGUAGE'} not used !)
	    
	$param->{'lang'} = $param->{'user'}{'lang'} || $list->{'admin'}{'lang'} 
	|| $Conf{'lang'};
	&POSIX::setlocale(&POSIX::LC_ALL, $locale_equiv{$param->{'lang'}});

	if ($comm{$action}) {
	    
	    $param->{'subtitle'} = $title{$action}{$param->{'lang'}};
	    $param->{'action'} = $action;
	    
            my $old_action = $action;

	    if ($action = &{$comm{$action}}()) {
		$param->{'active'} = 1;
	    }
	    
            unless ($action ne $old_action) {
		&wwslog('info','Stopping loop with %s action', $action);
		undef $action;
	    }

	    undef $action if ($action == 1);
	    
	    ## Reload from database if another action follows
#	    $param->{'user'} = &List::get_user_db($param->{'user'}{'email'})
#		if ($action && param->{'user'}{'email'});

	}elsif ($action) {
	    $param->{'error'}{'action'} = $action;
	    &message('unknown_action');
	    &wwslog('info','unknown action %s', $action);
	    last;
	}
    }

    ## Params 
    if ($param->{'list'}) {
	$param->{'title'} = "$param->{'list'}\@$param->{'host'}";
    }else {
	$param->{'title'} = $wwsconf->{'title'};
    }

    &set_menu();
    
    ## Set cookies
    if ($param->{'user'}{'email'}) {
	my $delay = $param->{'user'}{'cookie_delay'} || $wwsconf->{'cookie_expire'};
	&set_cookie($param->{'user'}{'email'}, $Conf{'cookie'}, $wwsconf->{'cookie_domain'},$delay ) || exit;
    }elsif ($ENV{'HTTP_COOKIE'}){
	&set_cookie('unknown',$Conf{'cookie'}, $wwsconf->{'cookie_domain'}, 'now');
    }
    
    if ($param->{'bypass'}) {
#	unless ($mime_types->{$param->{'file_extension'}}) {
#	    &do_log('debug', 'unknown MIME extension %s', $param->{'file_extension'});
#	    exit (-1);
#	}
	$mime_types->{$param->{'file_extension'}} ||= 'application/octet-stream';

	print "Cache-control: no-cache\n";
	print "Pragma: no-cache\n";
	printf "Content-Type: %s\n\n", $mime_types->{$param->{'file_extension'}};

	open (FILE, $param->{'file'});
	print <FILE>;
	close FILE;

    }elsif ($param->{'redirect_to'}) {
	print "Location: $param->{'redirect_to'}\n\n";
    }else {
	## Send HTML
	print "Cache-control: no-cache\n";
	print "Content-Type: text/html\n\n";
	
	## Icons
	$param->{'icons_url'} = $wwsconf->{'icons_url'};


	foreach my $tpldir ("$Conf{'home'}/$param->{'list'}/wws_templates","$Conf{'etc'}/wws_templates","/var/lib/sympa/bin/etc/wws_templates") {
	    if (-f "$tpldir/$param->{'action'}.$param->{'lang'}.tpl") {
		$param->{'action_template'} = "$tpldir/$param->{'action'}.$param->{'lang'}.tpl";
		last;
	    }
	    if (-f "$tpldir/$param->{'action'}.tpl") {
		$param->{'action_template'} = "$tpldir/$param->{'action'}.tpl";
		last;
	    }
	}
	unless ($param->{'action_template'})  {
	    &message('template_error');
	    &do_log('info',"unable to find template for $param->{'action'}");
	}

	## Menu template
	foreach my $tpldir ("$Conf{'home'}/$param->{'list'}/wws_templates","$Conf{'etc'}/wws_templates","/var/lib/sympa/bin/etc/wws_templates") {
	    if (-f "$tpldir/menu.$param->{'lang'}.tpl"){
		$param->{'menu_template'} = "$tpldir/menu.$param->{'lang'}.tpl";
		last;
	    }
	    if (-f "$tpldir/menu.tpl"){
		$param->{'menu_template'} = "$tpldir/menu.tpl";
		last;
	    }
	}
	unless ($param->{'menu_template'})  {
	    &message('template_error');
	    &do_log('info','unable to find menu template');
	}

	## Title template
	foreach my $tpldir ("$Conf{'home'}/$param->{'list'}/wws_templates","$Conf{'etc'}/wws_templates","/var/lib/sympa/bin/etc/wws_templates") {
	    if (-f "$tpldir/title.$param->{'lang'}.tpl"){
		$param->{'title_template'} = "$tpldir/title.$param->{'lang'}.tpl";
		last;
	    }
	    if (-f "$tpldir/title.tpl"){
		$param->{'title_template'} = "$tpldir/title.tpl";
		last;
	    }
	}
	unless ($param->{'title_template'})  {
	    &message('template_error');
	    &do_log('info','unable to find title template');
	}

	## Error template
	foreach my $tpldir ("$Conf{'home'}/$param->{'list'}/wws_templates","$Conf{'etc'}/wws_templates","/var/lib/sympa/bin/etc/wws_templates") {
	    if (-f "$tpldir/error.$param->{'lang'}.tpl"){
		$param->{'error_template'} = "$tpldir/error.$param->{'lang'}.tpl";
		last;
	    }
	    if (-f "$tpldir/error.tpl"){
		$param->{'error_template'} = "$tpldir/error.tpl";
		last;
	    }
	}
	unless ($param->{'error_template'})  {
	    &message('template_error');
	    &do_log('info','unable to find error template');
	}

	## Help template
	foreach my $tpldir ("$Conf{'home'}/$param->{'list'}/wws_templates","$Conf{'etc'}/wws_templates","/var/lib/sympa/bin/etc/wws_templates") {
	    if (-f "$tpldir/help_$param->{'help_topic'}.$param->{'lang'}.tpl"){
		$param->{'help_template'} = "$tpldir/help_$param->{'help_topic'}.$param->{'lang'}.tpl";
		last;
	    }
	    if (-f "$tpldir/help_$param->{'help_topic'}.tpl"){
		$param->{'help_template'} = "$tpldir/help_$param->{'help_topic'}.tpl";
		last;
	    }
	}

	## main template
	my $main ;
        foreach my $tpldir ("$Conf{'home'}/$param->{'list'}/wws_templates","$Conf{'etc'}/wws_templates","/var/lib/sympa/bin/etc/wws_templates") {
	    if (-f "$tpldir/main.$param->{'lang'}.tpl"){
		$main = "$tpldir/main.$param->{'lang'}.tpl";
		last;
	    }
	    if (-f "$tpldir/main.tpl"){
		$main = "$tpldir/main.tpl";
		last;
	    }
	}
	unless ($main)  {
	    &message('template_error');
	    &do_log('info','unable to find main template');
	}

	&parse_tpl($param,$main , STDOUT);
    }    

    # At the end of this loop reset variables is important to use this cgi as a CGI::fast 
    undef $param ; 
    
}

##############################################################
#-#\#|#/#-#\#|#/#-#\#|#/#-#\#|#/#-#\#|#/#-#\#|#/#-#\#|#/#-#\#|#/
##############################################################


sub new_loop {
    $loop++;
    my $query;

    if ($wwsconf->{'use_fast_cgi'}) {
	$query = new CGI::Fast;
    }else {	
	return undef if ($loop > 1);

	$query = new CGI;
    }
    
    return $query;
}

sub get_parameters {
#    &wwslog('debug', 'get_parameters');

    ## CGI URL
    if ($ENV{'HTTPS'} eq 'on') {
	$param->{'base_url'} = sprintf 'https://%s', $ENV{'HTTP_HOST'};
    }else {
	$param->{'base_url'} = sprintf 'http://%s', $ENV{'HTTP_HOST'};
    }

    if ($ENV{'REQUEST_METHOD'} eq 'GET') {
	my $path_info = $ENV{'PATH_INFO'};
	$path_info =~ s+^/++;
	my @params = split /\//, $path_info;
	
	## debug mode
	if ($params[0] eq 'debug') {
	    shift @params;
	    $Getopt::Std::opt_d = 1;
	}elsif ($params[0] eq 'debug2') {
	    shift @params;
	    $Getopt::Std::opt_d = 1;
	    $Getopt::Std::opt_D = 1;
	}
	
	if ($#params >= 0) {
	    $in{'action'} = $params[0];
	    
	    my $args;
	    if ($action_args{$in{'action'}}) {
		$args = $action_args{$in{'action'}};
	    }else {
		$args = $action_args{'default'};
	    }
	    
	    my $i = 1;
	    foreach my $p (@$args) {
		## More than 1 param
		if ($p =~ /^\@(\w+)$/) {
		    $p = $1;

		    $in{$p} = join '/', @params[$i..$#params];
		    last;
		}else {
		    $in{$p} = $params[$i];
		}
		$i++;
	    }
	}
    }elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
	## POST
	foreach my $p (keys %in) {
	    if ($p =~ /^action_(\w+)((\.\w+)*)$/) {
		
		$in{'action'} = $1;
		if ($2) {
		    foreach my $v (split /\./, $2) {
			$v =~ s/^\.?(\w+)\.?/$1/;
			$in{$v} = 1;
		    }
		}
		
		undef $in{$p};
	    }
	}
    }	
    
    ## Lowercase email addresses
    $in{'email'} = lc ($in{'email'});

    return 1;
}

## Analysis of parameters
sub check_param {
#    &wwslog('debug', 'check_param');

    ## Lowercase list name
    $in{'list'} =~ tr/A-Z/a-z/;

    if ($in{'list'}) {

	## Dev, for testing edit_list
	if ($in{'action'} =~ /^edit_list/) {
	    $ENV{'NEW_LOAD_ADMIN'} = 1;
	}

	unless ($list = new List ($in{'list'})) {
	    $param->{'error'}{'list'} = $in{'list'};
	    &message('unknown_list');
	    &wwslog('info','check_param: unknown list %s', $in{'list'});
	    return undef;
	}

	$param->{'list'} = $in{'list'};

	## privileges
	$param->{'is_subscriber'} = $list->is_user($param->{'user'}{'email'});
	$param->{'is_owner'} = $list->am_i('owner', $param->{'user'}{'email'});
	$param->{'is_editor'} = $list->am_i('editor', $param->{'user'}{'email'});
	$param->{'is_privileged_owner'} = $list->am_i('privileged_owner', $param->{'user'}{'email'});
	$param->{'is_priv'} = $param->{'is_owner'} || $param->{'is_editor'} || $param->{'is_listmaster'};
    }

    if ($param->{'user'}{'email'} && (&List::request_action ('create_list','md5',
							     {'sender' => $param->{'user'}{'email'},
							      'remote_host' => $param->{'remote_host'},
							      'remote_addr' => $param->{'remote_addr'}}) =~ /do_it|listmaster/)) {
	$param->{'may_create_list'} = 1;
    }else{
	undef ($param->{'may_create_list'});
    }
    ## listmaster has owner and editor privileges for the list
    if (&List::is_listmaster($param->{'user'}{'email'})) {
	$param->{'is_listmaster'} = 1;
	$param->{'is_owner'} = 1;
	$param->{'is_priv'} = 1;
    }

    ## Traces
#    foreach $role ('is_owner','is_editor','is_listmaster','is_priv') {
#	&wwslog('debug', "%s %s", $param->{'user'}{'email'}, $role) if $param->{$role} ;
#    }

    return 1;

}

## Define the menu 
sub set_menu {
#    &wwslog('debug', 'set_menu');
    
    my @menu = ('info','admin','none','home','pref','which','create_list_request',
		'serveradmin','logout','loginrequest','help');

    my $index = 0;

    my ($prev_status, $next_status) = ('none','none');
    foreach my $current (0..$#menu) {
	my $item = {'action' => $menu[$current]};

	if ($menu[$current] =~ /^loginrequest$/) {
	    next if $param->{'user'}{'email'};

 	}elsif ($menu[$current] =~ /^(pref|which|logout)$/) {
	    next unless ($param->{'user'}{'email'});

	}elsif ($menu[$current] =~ /^(info|admin)$/) {
	    next unless ($param->{'list'});
	    next if (($menu[$current] =~ /^admin$/) and
		     ! $param->{'is_priv'});
	    
	}elsif ($menu[$current] =~ /^edit_list$/) {
	    next unless ($param->{'may_create'});

	}elsif ($menu[$current] =~ /^serveradmin$/) {
	    next unless ($param->{'is_listmaster'} == 1);
	    
	}elsif ($menu[$current] =~ /^create_list_request$/) {
	    next unless ($param->{'may_create_list'});
	}
	
	if ($param->{'action'} eq $menu[$current]) {
	    $prev_status = $item->{'status'} = 'active';
	    
	    unless ($index == 0) {
		$param->{'menu'}[$index-1]{'next'} = 'active';
	    }

	}elsif ($menu[$current] eq 'none') {
	    $prev_status = $item->{'status'} = 'none';

	    unless ($index == 0) {
		$param->{'menu'}[$index-1]{'next'} = 'none';
	    }

	}else {
	    $prev_status = $item->{'status'} = 'passive';

	    unless ($index == 0) {
		$param->{'menu'}[$index-1]{'next'} = 'passive';
	    }

	}
	$item->{'previous'} = $prev_status;
	$item->{'title'} = $title{$menu[$current]}{$param->{'lang'}};

	$param->{'menu'}[$index] = $item;
	$index++;
    }
    $param->{'menu'}[$index-1]{'next'} = 'none';

#    unless ($none_inserted) {
#	$param->{'menu'}[$index] = {'status' =>'none',
#				    'title' => '',
#				    'previous' => $prev_status};
#    }
    
}

## Login WWSympa
sub do_login {
    &wwslog('debug', 'do_login(%s)', $in{'email'});
    my $user;
    my $next_action;
    
    if ($param->{'user'}{'email'}) {
	$param->{'error'}{'email'} = $param->{'user'}{'email'};
	&message('already_login');
	&wwslog('info','do_login: user %s already logged in', $param->{'user'}{'email'});
	return undef;
    }
    
    unless ($in{'email'}) {
	&message('no_email');
	&wwslog('info','do_login: no email');
	return undef;
    }
    
    unless (&valid_email($in{'email'})) {
	$param->{'error'}{'email'} = $in{'email'};
	&message('incorrect_email');
	&wwslog('info','do_login: incorrect email %s', $in{'email'});
	return undef;
    }
    
    
    unless ($in{'passwd'}) {
	&message('no_passwd');
	&wwslog('info','do_login: no passwd for user %s', $in{'email'});
	return undef;
    }
    
    unless ($user = &List::get_user_db($in{'email'})) {
	$param->{'error'}{'email'} = $in{'email'};
	&message('user_not_found');
	&wwslog('info','do_login: user %s not found', $in{'email'});
	return undef;
    }
    
    unless ($user->{'password'}) {
	$param->{'error'}{'email'} = $in{'email'};
	&message('passwd_not_found');
	&wwslog('info','do_login: password for user %s not found', $in{'email'});
	return undef;
    }
    
    unless ($in{'passwd'} eq $user->{'password'}) {
	&message('incorrect_passwd');
	&wwslog('info','do_login: incorrect password for user %s', $in{'email'});
	return undef;
    }
    
    $param->{'user'} = $user;
    
    if ($in{'referer'}) {
	$param->{'redirect_to'} = &tools::unescape_chars($in{'referer'});
    }elsif ($in{'previous_action'}) {
	$next_action = $in{'previous_action'};
	$in{'list'} = $in{'previous_list'};
    }else {
	$next_action = 'home';
    }
    
    return $next_action;
}

## send back login form
sub do_loginrequest {
    &wwslog('debug','do_loginrequest');
    
    if ($param->{'user'}{'email'}) {
	$param->{'error'}{'email'} = $param->{'user'}{'email'};
	&message('already_login');
	&wwslog('info','do_loginrequest: already logged in as %s', $param->{'user'}{'email'});
	return undef;
    }
    
    if ($in{'init_email'}) {
	$param->{'init_email'} = $in{'init_email'};
    }

    if ($in{'previous_action'} eq 'referer') {
	$param->{'referer'} = &tools::escape_chars($ENV{'HTTP_REFERER'});
    }elsif ($param->{'previous_action'}) {
	$param->{'previous_action'} = $in{'previous_action'};
	$param->{'previous_list'} = $in{'previous_list'};
    }

    return 1;
}

## Help / about WWSympa
sub do_help {
    &wwslog('debug','do_help(%s)', $in{'help_topic'});
    
    if ($in{'help_topic'}) {
	$param->{'no_menu'} = 1;
	$param->{'help_topic'} = $in{'help_topic'};
    }

    return 1;
}

## Logout from WWSympa
sub do_logout {
    &wwslog('debug','do_logout(%s)', $param->{'user'}{'email'});
    
    my $next_action;
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_logout: user not logged in');
	return undef;
    }
    
    delete $param->{'user'};
    
    &wwslog('info','do_logout: logout performed');
    
    if ($in{'previous_action'} eq 'referer') {
	$param->{'referer'} = &tools::escape_chars($in{'previous_list'});
    }

    return 'home';
}

## Remind the password
sub do_remindpasswd {
    &wwslog('debug', 'do_remindpasswd(%s)', $in{'email'}); 
    
    if ($in{'email'} and ! &valid_email($in{'email'})) {
	$param->{'error'}{'email'} = $in{'email'};
	&message('incorrect_email');
	&wwslog('info','do_remindpasswd: incorrect email %s', $in{'email'});
	return undef;
    }
    
    $param->{'email'} = $in{'email'};
    
    if ($in{'previous_action'} eq 'referer') {
	$param->{'referer'} = &tools::escape_chars($in{'previous_list'});
    }
    
    return 1;
}

sub do_sendpasswd {
    &wwslog('debug', 'do_sendpasswd(%s)', $in{'email'}); 
    my ($passwd, $user);
    
    unless ($in{'email'}) {
	&message('no_email');
	&wwslog('info','do_sendpasswd: no email');
	return undef;
    }
    
    unless (&valid_email($in{'email'})) {
	$param->{'error'}{'email'} = $in{'email'};
	&message('incorrect_email');
	&wwslog('info','do_sendpasswd: incorrect email %s', $in{'email'});
	return undef;
    }
    
    ## Init passwd if needed
    &init_passwd ($in{'email'}, {'lang' => $param->{'lang'}});

    $param->{'newuser'} =  &List::get_user_db($in{'email'});
    
    $param->{'newuser'}{'escaped_email'} =  &tools::escape_chars($param->{'newuser'}{'email'});

    $param->{'init_passwd'} = 1 
	if ($param->{'newuser'}{'password'} =~ /^INIT/);

    unless (open MAIL, "|$Conf{'sendmail'} $in{'email'}") {
	&message('mail_error');
	&wwslog('info','do_sendpasswd: mail error');
	return undef;
    }    

    my $tpl_file;
        
    foreach my $tpldir ("$Conf{'home'}/$param->{'list'}/wws_templates","$Conf{'etc'}/wws_templates","/var/lib/sympa/bin/etc/wws_templates") {
	if (-f "$tpldir/msg_sendpasswd.$param->{'lang'}.tpl") {
	    $tpl_file = "$tpldir/msg_sendpasswd.$param->{'lang'}.tpl";
	    last;
	}
	if (-f "$tpldir/msg_sendpasswd.tpl") {
	    $tpl_file = "$tpldir/msg_sendpasswd.tpl";
	    last;
	}
    }
    
    &parse_tpl ($param, $tpl_file, MAIL);
    close MAIL;

    $param->{'referer'} = $in{'referer'};
    
    if ($in{action} eq 'sendpasswd') {
	$param->{'init_email'} = $in{'email'};
	return 'loginrequest';
    }
    
    return 1;
}

## Which list the user is subscribed to 
## TODO (pour listmaster, toutes les listes)
sub do_which {
    my $which = {};
    my @lists;
    &wwslog('debug', 'do_which');
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_which: no user');
	return undef;
    }

    foreach $role ('member','owner','editor') {
	foreach $l ( &List::get_which($param->{'user'}{'email'}, $role) ) {
	    my $list = new List ($l);
	    
	    $param->{$role}{$l}{'subject'} = $list->{'admin'}{'subject'};
	    $param->{$role}{$l}{'host'} = $list->{'admin'}{'host'};
	}
    }

    return 1;
}

## The list of list
sub do_lists {
    my @lists;
    &wwslog('debug', 'do_lists(%s,%s)', $in{'topic'}, $in{'subtopic'});

    my $topics = &List::load_topics();

    if ($in{'topic'}) {
	if ($in{'subtopic'}) {
	    $param->{'subtitle'} = sprintf "%s / %s", $topics->{$in{'topic'}}{'title'}, $topics->{$in{'topic'}}{'sub'}{$in{'subtopic'}}{'title'};
	    $param->{'subtitle'} ||= "$in{'topic'} / $in{'subtopic'}";
	}else {
	    $param->{'subtitle'} = $topics->{$in{'topic'}}{'title'} || $in{'topic'};
	}
    }

    foreach $l ( &List::get_lists() ) {
	my $list = new List ($l);

	my $sender = $param->{'user'}{'email'} || 'nobody';
	my $action = &List::request_action ('visibility','md5',
			{'listname' =>  $l,
			 'sender' => $sender, 
			 'remote_host' => $param->{'remote_host'},
			 'remote_addr' => $param->{'remote_addr'}});

	next unless ($action eq 'do_it');
	
	my $list_info = {};
	$list_info->{'subject'} = $list->{'admin'}{'subject'};
	$list_info->{'host'} = $list->{'admin'}{'host'};

	if ($list->{'admin'}{'topics'}) {
	    foreach my $topic (@{$list->{'admin'}{'topics'}}) {
		my @tree = split '/', $topic;

		next if (($in{'topic'}) && ($tree[0] ne $in{'topic'}));
		next if (($in{'subtopic'}) && ($tree[1] ne $in{'subtopic'}));
		
		$param->{'lists'}{$list->{'name'}} = $list_info;
	    }
	}elsif ($in{'topic'} eq 'topicsless') {
	    $param->{'lists'}{$list->{'name'}} = $list_info;
	}
    }

    return 1;
}

## List information page
sub do_info {
    &wwslog('debug', 'do_info');

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_info: no list');
	return undef;
    }

    $param->{'subtitle'} = $list->{'admin'}{'subject'};
    $param->{'subscribe'} = $list->{'admin'}{'subscribe'}{'name'};
    $param->{'send'} = $list->{'admin'}{'send'}{'title'}{$param->{'lang'}};
    $param->{'total'} = $list->get_total();

    ## May review
    my $action = &List::request_action ('review','md5',
					{'listname' => $param->{'list'},
					 'sender' => $param->{'user'}{'email'},
					 'remote_host' => $param->{'remote_host'},
					 'remote_addr' => $param->{'remote_addr'}});
    
    $param->{'may_review'} = 1 if ($action =~ /do_it/);
    

    ## may subscribe
    $action = &List::request_action ('subscribe','md5',
				 {'listname' => $param->{'list'}, 
				  'sender' => $param->{'user'}{'email'},
				  'remote_host' => $param->{'remote_host'},
				  'remote_addr' => $param->{'remote_addr'}});
    
    $param->{'may_subscribe'} = 1 if ($action =~ /do_it|owner/);
        
    ## May signoff
    $action = &List::request_action ('unsubscribe','md5',
				     {'listname' =>$param->{'list'}, 
				      'sender' =>$param->{'user'}{'email'},
				      'remote_host' => $param->{'remote_host'},
				      'remote_addr' => $param->{'remote_addr'}});
    
    $param->{'may_signoff'} = 1 if ($action =~ /do_it|owner/);
    
    $param->{'may_subunsub'} = 1 if ($param->{'may_signoff'} || $param->{'may_subscribe'});

    ## Digest frequency
    if ($list->{'admin'}{'digest'} =~ /^([\d\,]+)\s+([\d\:]+)/m) {
	my (@days, $d);
	my $hour = $2;
	foreach $d (split /\,/, $1) {
	    push @days, $week{'fr'}[$d];
	}
	$param->{'digest'} = sprintf '%s - %s', (join ', ', @days), $hour;
    }
 
    ## Is_user
    if ($param->{'is_subscriber'}) {
	my ($s, $m);

	unless($s = $list->get_subscriber($param->{'user'}{'email'})) {
	    $param->{'error'}{'email'} = $param->{'user'}{'email'};
	    &message('subscriber_not_found');
	    &wwslog('info', 'do_info: subscriber %s not found', $param->{'user'}{'email'});
	    return undef;
	}

	$s->{'reception'} ||= 'mail';
	$s->{'visibility'} ||= 'noconceal';
	$s->{'date'} = &POSIX::strftime("%d %b %Y", localtime($s->{'date'}));
	
	foreach $m (keys %reception_mode) {
	    $param->{'reception'}{$m}{'description'} = $reception_mode{$m};
	    if ($s->{'reception'} eq $m) {
		$param->{'reception'}{$m}{'selected'} = 'SELECTED';
	    }else {
		$param->{'reception'}{$m}{'selected'} = '';
	    }
	}

	$param->{'subscriber'} = $s;
    }

    ## Get List Description
    if (-r "$Conf{'home'}/$param->{'list'}/homepage") {
	$param->{'homepage_file'} = "$Conf{'home'}/$param->{'list'}/homepage";
    }else {
	$param->{'info_file'} = "$Conf{'home'}/$param->{'list'}/info";
    }
    ## Owners
    foreach $o (@{$list->{'admin'}{'owner'}}) {
	$param->{'owner'}{$o->{'email'}}{'gecos'} = $o->{'gecos'} || $o->{'email'};
    }

    ## Editors
    $param->{'is_moderated'} = $list->is_moderated();
    foreach $e (@{$list->{'admin'}{'editor'}}) {
	$param->{'editor'}{$e->{'email'}}{'gecos'} = $e->{'gecos'} || $e->{'email'};
    }  

    ## Archives Access control
    if ($list->{'admin'}{'web_archive'}{'defined'}) {
	$param->{'is_archived'} = 1;
	&wwslog('info','request_action (access_web_archive,md5,%s,%s,%s)',$param->{'list'},$param->{'user'}{'email'},$param->{'remote_host'}) ; 
	if (&List::request_action ('access_web_archive','md5',
				   {'listname' => $param->{'list'},
				    'sender' => $param->{'user'}{'email'},
				    'remote_host' => $param->{'remote_host'},
				    'remote_addr' => $param->{'remote_addr'}}) =~ /do_it/i) {
	    $param->{'arc_access'} = 1; 
	}else{
	    undef ($param->{'arc_access'});
	}
    }
    return 1;
}

## Subscribers' list
sub do_review {
    &wwslog('debug', 'do_review(%d)', $in{'page'});
    my $record;
    my @users;
    my $size = $in{'size'} || $wwsconf->{'review_page_size'};
    my $sortby = $in{'sortby'} || 'email';

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_review: no list');
	return undef;
    }

    ## May review
    my $action = &List::request_action ('review','md5',
					{'listname' => $param->{'list'},
					 'sender' => $param->{'user'}{'email'},
					 'remote_host' => $param->{'remote_host'},
					 'remote_addr' => $param->{'remote_addr'}});
    unless ($action =~ /do_it/) {
	$param->{'error'}{'action'} = 'review';
	&message('may_not');
	&wwslog('info','do_review: may not review');
	return undef;
    }

    unless ($list->get_total()) {
	&message('no_subscriber');
	&wwslog('info','do_review: no subscriber');
	return undef;
    }

    ## Owner
    $param->{'total'} = $list->get_total();
    $param->{'page'} = $in{'page'} || 1;
    $param->{'total_page'} = int ($param->{'total'} / $size);
    $param->{'total_page'} ++
	if ($param->{'total'} % $size);

    if ($param->{'page'} > $param->{'total_page'}) {
	$param->{'error'}{'page'} = $param->{'page'};
	&message('no_page');
	&wwslog('info','do_review: no page %d', $param->{'page'});
	return undef;
    }

    my $offset;
    if ($param->{'page'} > 1) {
	$offset = (($param->{'page'} - 1) * $size);
    }else {
	$offset = 0;
    }

    ## Members list
    for ($i = $list->get_first_user($sortby, $offset, $size); 
	 $i; $i = $list->get_next_user()) {
	next if (($i->{'visibility'} eq 'conceal')
		 and (! $param->{'is_owner'}) );
	
	## Add user
	$i->{'date'} = &POSIX::strftime("%d %b %Y", localtime($i->{'date'}));

	$i->{'reception'} ||= 'mail';

	## Escape some weird chars
	$i->{'escaped_email'} = &tools::escape_chars($i->{'email'});

	push @{$param->{'members'}}, $i;
    }

    if ($param->{'page'} > 1) {
	$param->{'prev_page'} = $param->{'page'} - 1;
    }

    unless (($offset + $size) >= $param->{'total'}) {
	$param->{'next_page'} = $param->{'page'} + 1;
    }

    $param->{'size'} = $size;
    $param->{'sortby'} = $sortby;

    return 1;
}

## Search in subscribers
sub do_search {
    &wwslog('debug', 'do_search(%s)', $in{'filter'});

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_search: no list');
	return undef;
    }

    unless ($in{'filter'}) {
	&message('no_filter');
	&wwslog('info','do_search: no filter');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_search: no user');
	return undef;
    }

    ## May review
    my $sender = $param->{'user'}{'email'} || 'nobody';
    my $action = &List::request_action ('review','md5',
					{'listname' => $param->{'list'},
					 'sender' => $sender,
					 'remote_host' => $param->{'remote_host'},
					 'remote_addr' => $param->{'remote_addr'}});

    unless ($action =~ /do_it/) {
	$param->{'error'}{'action'} = 'review';
	&message('may_not');
	&wwslog('info','do_search: may not review');
	return undef;
    }

    ## Regexp
    $param->{'filter'} = $in{'filter'};
    $param->{'regexp'} = $param->{'filter'};
    $param->{'regexp'} =~ s/\\/\\\\/g;
    $param->{'regexp'} =~ s/\./\\\./g;
    $param->{'regexp'} =~ s/\*/\.\*/g;
    $param->{'regexp'} =~ s/\+/\\\+/g;
    $param->{'regexp'} =~ s/\?/\\\?/g;

    my $record = 0;
    ## Members list
    for ($i = $list->get_first_user(); $i; $i = $list->get_next_user()) {

	## Search filter
	next if ($i->{'email'} !~ /$param->{'regexp'}/i);

	next if (($i->{'visibility'} eq 'conceal')
		 and (! $param->{'is_owner'}) );
	
	## Add user
	$i->{'date'} = &POSIX::strftime("%d %b %Y", localtime($i->{'date'}));

	$i->{'reception'} ||= 'mail';

	## Escape some weird chars
	$i->{'escaped_email'} = &tools::escape_chars($i->{'email'});

	$record++;
	push @{$param->{'members'}}, $i;
    }

    $param->{'occurrence'} = $record;
    return 1;
}

## Access to user preferences
sub do_pref {
    &wwslog('debug', 'do_pref');

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_pref: no user');
	return undef;
    }

    ## Available languages
    foreach $l (keys %languages) {
	$param->{'languages'}{$l}{'complete'} = $languages{$l};
	if ($param->{'lang'} eq $l) {
	    $param->{'languages'}{$l}{'selected'} = 'SELECTED';
	}else {
	    $param->{'languages'}{$l}{'selected'} = '';
	}
    }
    
    return 1;
}

## Set the initial password
sub do_choosepasswd {
    &wwslog('debug', 'do_choosepasswd');

    unless ($param->{'user'}{'email'}) {
	unless ($in{'email'} && $in{'passwd'}) {
	    &message('no_user');
	    &wwslog('info','do_pref: no user');
	    return undef;
	}

	$in{'previous_action'} = 'choosepasswd';
	return 'login';
    }

    $param->{'init_passwd'} = 1 if ($param->{'user'}{'password'} =~ /^INIT/);

    return 1;
}

## Change subscription parameter
sub do_set {
    &wwslog('debug', 'do_set(%s, %s)', $in{'reception'}, $in{'gecos'});

    my $reception;
    my $email;

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_set: no list');
	return undef;
    }
    
    unless ($reception = $in{'reception'}) {
	&message('no_reception');
	&wwslog('info','do_set: no reception');
	return undef;
    }
    
    if ($in{'email'}) {
    	unless ($param->{'is_owner'}) {
	    $param->{'error'}{'action'} = 'set';
	    &message('may_not');
	    &wwslog('info','do_set: not owner');
	    return undef;
        }
	
	$email = &tools::unescape_chars($in{'email'});
    }else {
    	unless ($param->{'user'}{'email'}) {
	    &message('no_user');
	    &wwslog('info','do_set: no user');
	    return undef;
        }
	$email = $param->{'user'}{'email'};
    } 
    
    unless ($list->is_user($email)) {
	&message('not_subscriber');
	&wwslog('info','do_set: %s not subscriber of list %s', $email, $param->{'list'});
	return undef;
    }
    
    $reception = '' if $reception eq 'mail';
    
    my $update = {'reception' => $reception};
    
    $update->{'gecos'} = $in{'gecos'} if $in{'gecos'};
    
    unless ( $list->update_user($email, $update) ) {
	&message('failed');
	&wwslog('info', 'do_set: set failed');
	return undef;
    }
    
    $param->{'error'}{'action'} = 'set';
    &message('performed');
    
    return 1;
}

## Update of user preferences
sub do_setpref {
    &wwslog('debug', 'do_setpref');
    my $changes = {};

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_pref: no user');
	return undef;
    }

    foreach $p ('gecos','lang','cookie_delay') {
	$changes->{$p} = $in{$p};
    }

    unless (&List::update_user_db($param->{'user'}{'email'}, $changes)) {
	&message('update_failed');
	&wwslog('info','do_pref: update failed');
	return undef;
    }

    foreach $p ('gecos','lang','cookie_delay') {
	$param->{'user'}{$p} = $in{$p};
    }


    return 'pref';
}

## Prendre en compte les dfauts
sub do_viewfile {
    &wwslog('debug', 'do_viewfile');

    unless ($in{'file'}) {
	$param->{'error'}{'argument'} = 'file';
	&message('missing_arg');
	&wwslog('info','do_viewfile: no file');
	return undef;
    }

    unless (defined $filenames{$in{'file'}}) {
	$param->{'error'}{'file'} = $in{'file'};
	&message('file_not_editable');
	&wwslog('info','do_viewfile: file %s not editable', $in{'file'});
	return undef;
    }

   unless ($param->{'list'}) {
       $param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_viewfile: no list');
	return undef;
    }

    $param->{'file'} = $in{'file'};
    $param->{'subtitle'} = sprintf $param->{'subtitle'}, $filenames{$in{'file'}}{$param->{'lang'}};

    $param->{'filepath'} = "$Conf{'home'}/$list->{'name'}/$in{'file'}";

    if ((-e $param->{'filepath'}) and (! -r $param->{'filepath'})) {
	&message('read_error');
	&wwslog('info','do_viewfile: cannot read %s', $param->{'filepath'});
	return undef;
    }

    return 1;
}

## Subscribe to the list
## TOTO: accepter nouveaux users
sub do_subscribe {
    &wwslog('debug', 'do_subscribe(%s)', $in{'email'});

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_subscribe: no list');
	return undef;
    }

    unless ($param->{'user'}{'email'}) {
	unless ($in{'email'}) {
	    &message('no_user');
	    &wwslog('info','do_subscribe: no user');
	    return undef;
	}

	## Perform login
	if ($in{'passwd'}) {
	    $in{'previous_action'} = 'subscribe';
	    $in{'previous_list'} = $param->{'list'};
	    return 'login';
	}

	if ( &List::is_user_db($in{'email'}) ) {
	    &message('no_user');
	    &wwslog('info','do_subscribe: need auth for user %s', $in{'email'});
	    return undef;
	}
	
	## Here is a brand new user
	&init_passwd($in{'email'},{'lang' => $param->{'lang'},
				   'gecos' => $in{'gecos'}});
	
#	$param->{'user'}{'email'} = &List::get_user_db($in{'email'});
    }

    if ( $list->is_user($param->{'user'}{'email'}) ) {
	$param->{'error'}{'list'} = $list->{'name'};
	&message('already_subscriber');
	&wwslog('info','do_subscribe: %s already subscriber', $param->{'user'}{'email'});
	return undef;
    }
	
    my $sub_is = &List::request_action('subscribe','md5',
				       {'listname' => $param->{'list'},
					'sender' => $param->{'user'}{'email'}, 
					'remote_host' => $param->{'remote_host'},
					'remote_addr' => $param->{'remote_addr'}});

    if ($sub_is eq 'closed') {
	$param->{'error'}{'action'} = 'subscribe';
       	&message('may_not');
	&wwslog('info', 'do_subscribe: subscribe closed');
	return undef;
    }

    $param->{'may_subscribe'} = 1;
    
    if ($sub_is eq 'owner') {
	$list->send_sub_to_owner($param->{'user'}{'email'}, $param->{'user'}{'gecos'});
	&message('sent_to_owner');
	&wwslog('info', 'do_subscribe: subscribe sent to owner');

	return 'info';
    }elsif ($sub_is =~ /do_it/) {
	my $u = $list->get_default_user_options();
	$u->{'email'} = $param->{'user'}{'email'};
	$u->{'gecos'} = $param->{'user'}{'gecos'};
	$u->{'date'} = time;
	
	unless ($list->add_user($u)) {
	    &message('failed');
	    &wwslog('info', 'do_subscribe: subscribe failed');
	    return undef;
	}

	$list->save();

	$list->send_file('welcome', $param->{'user'}{'email'});
	if ($sub_is =~ /notify/) {
	    $list->send_notify_to_owner($param->{'user'}{'email'}, $param->{'user'}{'gecos'}, 'subscribe');
	}
    }
    
    $param->{'error'}{'action'} = 'subscribe';
    &message('performed');

    return 'info';
}

## Subscription request (user not authentified)
sub do_subrequest {
    &wwslog('debug', 'do_subrequest(%s)', $in{'email'});
    
    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_subrequest: no list');
	return undef;
    }
    
    ## Not auth & no email
    unless ($param->{'user'}{'email'} or $in{'email'}) {
	&message('no_email');
	&wwslog('info','do_subrequest: no email');
	return undef;
    }
    
    ## Basic check of email if provided
    if ($in{'email'}) {
	unless (&valid_email($in{'email'})) {
	    &message('incorrect_email');
	    &wwslog('info','do_subrequest: incorrect email %s'
		    , $in{'email'});
	    return undef;
	}
    }
    
    ## Subscriber (auth or not)
    if ($param->{'is_subscriber'} ) {
	&message('already_subscriber');
	&wwslog('info','do_subrequest: user %s is already subscriber', $param->{'user'}{'email'} || $in{'email'});
	return undef;
    } 
    
    unless ($param->{'user'}{'email'}) {
	if ($list->is_user($in{'email'})) {
	    $param->{'subscriber_not_auth'} = 1;
	}else {
   	    unless (&init_passwd ($in{'email'}, {'lang' => $param->{'lang'},
						 'gecos' => $in{'gecos'}})) {
	 	&message('failed');
         	&wwslog('info','subrequest: password initialization for user %s failed', $in{'email'});
         	return undef;
    	    }

	    my $user = &List::get_user_db($in{'email'});

	    if ($user->{'password'} =~ /^INIT/) {
		&do_sendpasswd() if ($user->{'password'} =~ /^INIT/);
		$param->{'init_passwd'} = 1;
	    }
	}
    }
    
    $param->{'subscriber_not_auth'} = 1 
	if (!$param->{'user'}{'email'} && $list->is_user($in{'email'}));
    
    $param->{'email'} = $in{'email'};

    return 1;
}

## Unsubscribe from list
sub do_signoff {
    &wwslog('debug', 'do_signoff');

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_signoff: no list');
	return undef;
    }

    unless ($param->{'user'}{'email'}) {
	unless ($in{'email'}) {
	    &message('no_user');
	    &wwslog('info','do_signoff: no user');
	    return undef;
	}

	## Perform login first
	if ($in{'passwd'}) {
	    $in{'previous_action'} = 'signoff';
	    $in{'previous_list'} = $param->{'list'};
	    return 'login';
	}
	
	if ( &List::is_user_db($in{'email'}) ) {
	    &message('no_user');
	    &wwslog('info','do_signoff: need auth for user %s', $in{'email'});
	    return undef;
	}
	
	## No passwd
	&init_passwd($in{'email'}, {'lang' => $param->{'lang'} });
	
	$param->{'user'}{'email'} = $in{'email'};
    }
    
    unless ($list->is_user($param->{'user'}{'email'})) {
	$param->{'error'}{'list'} = $list->{'name'};
	&message('not_subscribed');
	&wwslog('info','do_signoff: %s not subscribed to %s',$param->{'user'}{'email'}, $param->{'list'} );
	return undef;
    }

    my $sig_is = &List::request_action ('unsubscribe','md5',
					{'listname' => $param->{'list'}, 
					 'sender' => $param->{'user'}{'email'},
					 'remote_host' => $param->{'remote_host'},
					 'remote_addr' => $param->{'remote_addr'}});

    $param->{'may_signoff'} = 1 if ($sig_is =~ /do_it|owner/);
    
    if ($sig_is =~ /reject/) {
	$param->{'error'}{'action'} = 'unsubscribe';
	&message('may_not');
	&wwslog('info', 'do_signoff: %s may not signoff from %s'
		, $param->{'user'}{'email'}, $param->{'list'});
	return undef;
    }elsif ($sig_is =~ /owner/) {
	$list->send_sig_to_owner($param->{'user'}{'email'});
	&message('sent_to_owner');
	&wwslog('info', 'do_signoff: signoff sent to owner');
	return undef;
    }else {
	unless ($list->delete_user($param->{'user'}{'email'})) {
	    &message('failed');
	    &wwslog('info', 'do_signoff: signoff failed');
	    return undef;
	}

	$list->save();

	if ($sig_is =~ /notify/) {
	    $list->send_notify_to_owner($param->{'user'}{'email'}, '', 'signoff');
	}
	
	$list->send_file('bye', $param->{'user'}{'email'});
    }

    $param->{'error'}{'action'} = 'unsubscribe';
    &message('performed');

    return 'info';
}

## Unsubscription request (user not authentified)
sub do_sigrequest {
    &wwslog('debug', 'do_sigrequest(%s)', $in{'email'});
    
    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_sigrequest: no list');
	return undef;
    }

    ## Do it
    if ($param->{'user'}{'email'}) {
	return 'signoff';
    }
    
    ## Not auth & no email
    unless ($in{'email'}) {
	&message('no_email');
	&wwslog('info','do_sigrequest: no email');
	return undef;
    }
    
    ## Basic check of email if provided
    if ($in{'email'}) {
	unless (&valid_email($in{'email'})) {
	    &message('incorrect_email');
	    &wwslog('info','do_sigrequest: incorrect email %s'
		    , $in{'email'});
	    return undef;
	}
    }
    
    if ($list->is_user($in{'email'})) {
	unless (&init_passwd ($in{'email'}, {'lang' => $param->{'lang'} })) {
	    &message('failed');
	    &wwslog('info','sigrequest: password initialization for user %s failed', $in{'email'});
	    return undef;
	}
	
	my $user = &List::get_user_db($in{'email'});
	
	if ($user->{'password'} =~ /^INIT/) {
	    &do_sendpasswd() if ($user->{'password'} =~ /^INIT/);
	    $param->{'init_passwd'} = 1;
	}
    }else {
	$param->{'not_subscriber'} = 1;
    }
    
    $param->{'email'} = $in{'email'};
    
    return 1;
}


## Update of password
sub do_setpasswd {
    &wwslog('debug', 'do_setpasswd');
    my $user;

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_setpasswd: no user');
	return undef;
    }

    unless ($user = &List::get_user_db($param->{'user'}{'email'})) {
	$param{'error'}{'email'} = $param->{'user'}{'email'};
	&message('user_not_found');
	&wwslog('info','setpasswd: user %s not found', $param->{'user'}{'email'});
	return undef;
    }

    unless ($in{'passwd'}) {
	&message('no_passwd');
	&wwslog('info','do_setpasswd: no passwd');
	return undef;
    }
  
    unless ($in{'newpasswd1'}) {
	&message('no_passwd');
	&wwslog('info','do_setpasswd: no newpasswd1');
	return undef;
    }
  
    unless ($in{'newpasswd2'}) {
	&message('no_passwd');
	&wwslog('info','do_setpasswd: no newpasswd2');
	return undef;
    }

    unless ($in{'newpasswd1'} eq $in{'newpasswd2'}) {
	&message('diff_passwd');
	&wwslog('info','do_setpasswd: different newpasswds');
	return undef;
    }
  
    unless ( $in{'passwd'} eq $user->{'password'}) {
	&message('incorrect_passwd');
	&wwslog('info','setpasswd: incorrect password for user %s', $param->{'user'}{'email'});
	return undef;
    }

    unless ( &List::update_user_db($param->{'user'}{'email'}, {'password' => $in{'newpasswd1'}} )) {
	&message('failed');
	&wwslog('info','do_setpasswd: update failed');
	return undef;
    }

    $param->{'user'}{'password'} =  $in{'newpasswd1'};

    $param->{'error'}{'action'} = 'setpassword';
    &message('performed');

    if ($in{'previous_action'}) {
	$in{'list'} = $in{'previous_list'};
	return $in{'previous_action'};
    }else {
	return 'pref';
    }
}

## List admin page
sub do_admin {
    &wwslog('debug', 'do_admin');

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_admin: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_admin: no user');
	return undef;
    }
 
    unless ($param->{'is_owner'} or $param->{'is_editor'}) {
	$param->{'error'}{'action'} = 'admin';
	&message('may_not');
	&wwslog('info','do_admin: %s not priv user', $param->{'user'}{'email'});
	return undef;
    }
 
    ## Messages edition
    foreach $f ('info','homepage','welcome.tpl','bye.tpl','removed.tpl','message.footer','message.header','remind.tpl','invite.tpl','reject.tpl') {
	next unless ($list->may_edit($f, $param->{'user'}{'email'}) eq 'write');
	$param->{'files'}{$f}{'complete'} = $filenames{$f}{$param->{'lang'}};
	$param->{'files'}{$f}{'selected'} = '';
    }
    $param->{'files'}{'info'}{'selected'} = 'SELECTED';
   
    $param->{'total'} = $list->get_total();
    $param->{'mod_total'} = $list->get_mod_spool_size();
    
    my $total = $param->{'total'};
    if ($total > 0) {
	$param->{'bounce_rate'} = $list->get_total_bouncing() * 100 / $total;
	$param->{'bounce_rate'} = int ($param->{'bounce_rate'} * 10) / 10;
    }
    return 1;
}

## Server admin page
sub do_serveradmin {
    &wwslog('debug', 'do_serveradmin');
    my $f;

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_serveradmin: no user');
	return undef;
    }
 
    unless ($param->{'is_listmaster'}) {
	$param->{'error'}{'action'} = 'serveradmin';
	&message('may_not');
	&wwslog('info','do_admin: %s not listmaster', $param->{'user'}{'email'});
	return undef;
    }
 
    $param->{'conf'} = \%Conf;

    ## Lists Default files
    foreach $f ('welcome.tpl','bye.tpl','removed.tpl','message.footer','message.header','remind.tpl') {
	$param->{'lists_default_files'}{$f}{'complete'} = $filenames{$f}{$param->{'lang'}};
	$param->{'lists_default_files'}{$f}{'selected'} = '';
    }
    
    ## Server files
    foreach $f ('helpfile','helpfile.advanced','lists.header','lists.footer') {
	$param->{'server_files'}{$f}{'complete'} = $filenames{$f}{$param->{'lang'}};
	$param->{'server_files'}{$f}{'selected'} = '';
    }
    $param->{'server_files'}{'helpfile'}{'selected'} = 'SELECTED';
    

    return 1;
}

## Multiple add
sub do_add_request {
    &wwslog('debug', 'do_add_request(%s)', $in{'email'});

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_add_request: no list');
	return undef;
    }
 
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_add_request: no user');
	return undef;
    }

    my $add_is = &List::request_action ('add','md5',
					{'listname' => $param->{'list'},
					 'sender' => $param->{'user'}{'email'}, 
					 'email' => 'nobody',
					 'remote_host' => $param->{'remote_host'},
					 'remote_addr' => $param->{'remote_addr'}});

    unless ($add_is =~ /do_it/) {
	$param->{'error'}{'action'} = 'add';
	&message('may_not');
	&wwslog('info','do_add_request: %s may not add', $param->{'user'}{'email'});
	return undef;
    }

    return 1;
}
## Add a user to a list
## TODO: vrifier validit email
sub do_add {
    &wwslog('debug', 'do_add(%s)', $in{'email'});

    my %user;

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_add: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_add: no user');
	return undef;
    }
 
    my $add_is = &List::request_action ('add','md5',
					{'listname' => $param->{'list'},
					 'sender' => $param->{'user'}{'email'}, 
					 'email' => $in{'email'},
					 'remote_host' => $param->{'remote_host'},
					 'remote_addr' => $param->{'remote_addr'}});

    unless ($add_is =~ /do_it/) {
	$param->{'error'}{'action'} = 'add';
	&message('may_not');
	&wwslog('info','do_add: %s may not add', $param->{'user'}{'email'});
	return undef;
    }
    
    if ($in{'dump'}) {
	foreach (split /\n/, $in{'dump'}) {
	    if (/^(\S+|\".*\"@\S+)(\s+(.*))?\s*$/) {
		$user{$1} = $3;
	    }
	}
    }elsif ($in{'email'}) {
	$user{$in{'email'}} = $in{'gecos'};
    }else {
	&message('no_email');
	&wwslog('info','do_add: no email');
	return undef;
    }

    foreach my $email (keys %user) {

	unless (&valid_email($email)) {
	    $param->{'error'}{'email'} = $email;
	    &message('incorrect_email');
	    &wwslog('info','do_add: incorrect email %s', $email);
	    return undef;
	}

	if ( $list->is_user($email) ) {
	    $param->{'error'}{'email'} = $email;
	    $param->{'error'}{'list'} = $list->{'name'};
	    &message('user_already_subscriber');
	    &wwslog('info','do_add: %s already subscriber', $email);
	    return undef;
	}
    
	my $u = $list->get_default_user_options();
	$u->{'email'} = $email;
	$u->{'gecos'} = $user{$email};
	$u->{'date'} = time;
	unless( $list->add_user($u)) {
	    $param->{'error'}{'action'} = 'add';
	    &message('failed');
	    &wwslog('info','do_add: failed');
	    return undef;
	}
	
	$list->save();
	
	$list->send_file('welcome', $email) unless ($in{'quiet'});
    }
    
    $param->{'error'}{'action'} = 'add';
    &message('performed');

    return $in{'previous_action'} || 'admin';
}

## Del a user to a list
## TODO: vrifier validit email
sub do_del {
    &wwslog('debug', 'do_del(%s)', $in{'email'});

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_del: no list');
	return undef;
    }
    
    unless ($in{'email'}) {
	&message('no_email');
	&wwslog('info','do_del: no email');
	return undef;
    }

    $in{'email'} = &tools::unescape_chars($in{'email'});

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_del: no user');
	return undef;
    }
 
    my $del_is = &List::request_action ('del','md5',
					{'listname' =>$param->{'list'},
					 'sender' => $param->{'user'}{'email'},
					 'email' => $in{'email'},
					 'remote_host' => $param->{'remote_host'},
					 'remote_addr' => $param->{'remote_addr'}});

    unless ( $del_is =~ /do_it/) {
	$param->{'error'}{'action'} = 'del';
	&message('may_not');
	&wwslog('info','do_del: %s may not del', $param->{'user'}{'email'});
	return undef;
    }

    my @emails = split /\0/, $in{'email'};

    foreach my $email (@emails) {
    
	unless ( $list->is_user($email) ) {
	    &message('not_subscribed');
	    &wwslog('info','do_del: %s not subscribed', $email);
	    return undef;
	}
	
	unless( $list->delete_user($email)) {
	    $param->{'error'}{'action'} = 'del';
	    &message('failed');
	    &wwslog('info','do_del: failed for %s', $email);
	    return undef;
	}

	&wwslog('info','do_del: subscriber %s deleted from list %s', $email, $param->{'list'});
	
	$list->save();

	$list->send_file('removed', $email) unless ($in{'quiet'});	
    }

    $param->{'error'}{'action'} = 'del';
    &message('performed');
    
    return $in{'previous_action'} || 'review';
}

sub do_modindex {
    &wwslog('debug', 'do_modindex');
    my $msg;

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_modindex: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_modindex: no user');
	return undef;
    }
 
    unless ($list->am_i('editor', $param->{'user'}{'email'})) {
	$param->{'error'}{'action'} = 'modindex';
	&message('may_not');
	&wwslog('info','do_modindex: %s not editor', $param->{'user'}{'email'});
	return undef;
    }

    ## Loads message list
    unless (opendir SPOOL, $Conf{'queuemod'}) {
	&message('spool_error');
	&wwslog('info','do_modindex: unable to read spool');
	return undef;
    }
    
    foreach $msg ( sort grep(!/^\./, readdir SPOOL )) {
	next
	    unless ($msg =~ /^$list->{'name'}\_(\w+)$/);
	
	my $id = $1;

	## Load msg
	unless (open MSG, "$Conf{'queuemod'}/$msg") {
	    &message('msg_error');
	    &wwslog('info','do_modindex: unable to read msg %s', $msg);
	    return undef;
	}

	my $mail = new Mail::Internet [<MSG>];
	close MSG;

	$param->{'spool'}{$id}{'size'} = int( (-s "$Conf{'queuemod'}/$msg") / 1024 + 0.5);
	$param->{'spool'}{$id}{'subject'} =  &MIME::Words::decode_mimewords($mail->head->get('Subject'));
	$param->{'spool'}{$id}{'subject'} ||= 'no_subject';
	$param->{'spool'}{$id}{'date'} = $mail->head->get('Date');
	$param->{'spool'}{$id}{'from'} = &MIME::Words::decode_mimewords($mail->head->get('From'));
	$param->{'spool'}{$id}{'from'} =~ s/</&lt;/;
	$param->{'spool'}{$id}{'from'} =~ s/>/&gt;/;
    }

    unless ($param->{'spool'}) {
	&message('no_msg');
	&wwslog('info','do_modindex: no message');
	return undef;
    }


    return 1;
}

sub do_reject {
    &wwslog('debug', 'do_reject(%s)', $in{'id'});
    my ($msg, $file);

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_reject: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_reject: no user');
	return undef;
    }
 
    unless ($list->am_i('editor', $param->{'user'}{'email'})) {
	$param->{'error'}{'action'} = 'reject';
	&message('may_not');
	&wwslog('info','do_reject: %s not editor', $param->{'user'}{'email'});
	return undef;
    }

    unless ($in{'id'}) {
	$param->{'error'}{'argument'} = 'msgid';
	&message('missing_arg');
	&wwslog('info','do_reject: no msgid');
	return undef;
    }
   
    foreach my $id (split /\0/, $in{'id'}) {

	$file = "$Conf{'queuemod'}/$list->{'name'}_$id";

	## Open the file
	if (!open(IN, $file)) {
	    &message('failed_someone_else_did_it');
	    &wwslog('info','do_reject: Unable to open %s', $file);
	    return undef;
	}
	unless ($in{'quiet'}) {
	    my $message = new Mail::Internet [<IN>];
	    my @sender_hdr = Mail::Address->parse($message->head->get('From'));
	    unless  ($#sender_hdr == -1) {
		my $rejected_sender = $sender_hdr[0]->address;
		my %context;
		$context{'subject'} = $message->head->get('subject');
		$context{'rejected_by'} = $sender;
		$list->send_file('reject', $rejected_sender, \%context);
	    }
	}
	close(IN);  
	
	unless (unlink($file)) {
	    $param->{'error'}{'action'} = 'reject';
	    &message('failed');
	    &wwslog('info','do_reject: failed to erase %s', $file);
	    return undef;
	}
	
    }

    $param->{'error'}{'action'} = 'reject';
    &message('performed');
    
    return 'modindex';
}

## TODO: supprimer le msg
sub do_distribute {
    &wwslog('debug', 'do_distribute(%s)', $in{'id'});
    my ($msg, $file);

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_distribute: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_distribute: no user');
	return undef;
    }
    
    unless ($list->am_i('editor', $param->{'user'}{'email'})) {
	$param->{'error'}{'action'} = 'distribute';
	&message('may_not');
	&wwslog('info','do_distribute: %s not editor', $param->{'user'}{'email'});
	return undef;
    }

    unless ($in{'id'}) {
	$param->{'error'}{'argument'} = 'msgid';
	&message('missing_arg');
	&wwslog('info','do_distribute: no msgid');
	return undef;
    }
    my $extention = time.".".int(rand 9999) ;
    open DISTRIBUTE, ">$Conf{'queue'}/T.$Conf{'sympa'}.$extention" ;

    printf DISTRIBUTE ("X-Sympa-To: %s\n",$Conf{'sympa'});
    printf DISTRIBUTE ("Message-Id: <%s\@wwsympa>\n",$time);
    printf DISTRIBUTE ("From: %s\n\n", $param->{'user'}{'email'});

    foreach my $id (split /\0/, $in{'id'}) {
	
	$file = "$Conf{'queuemod'}/$list->{'name'}_$id";

	printf DISTRIBUTE ("QUIET DISTRIBUTE %s %s\n",$list->{'name'},$id);
	unless (rename($file,"$file.distribute")) {
	    $param->{'error'}{'action'} = 'distribute';
	    &message('failed');
	    &wwslog('info','do_distribute: failed to rename %s', $file);
	}


    }
    close DISTRIBUTE;
    rename("$Conf{'queue'}/T.$Conf{'sympa'}.$extention","$Conf{'queue'}/$Conf{'sympa'}.$extention");

    $param->{'error'}{'action'} = 'distribute';
    &message('performed_soon');
    
    return 'modindex';
}

sub do_viewmod_old {
    &wwslog('debug', 'do_viewmod(%s)', $in{'id'});
    my $msg;

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_viewmod: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_viewmod: no user');
	return undef;
    }
 
    unless ($in{'id'}) {
	$param->{'error'}{'argument'} = 'msgid';
	&message('missing_arg');
	&wwslog('info','do_viewmod: no msgid');
	return undef;
    }
   
    unless ($list->am_i('editor', $param->{'user'}{'email'})) {
	$param->{'error'}{'action'} = 'viewmod';
	&message('may_not');
	&wwslog('info','do_viewmod: %s not editor', $param->{'user'}{'email'});
	return undef;
    }

    ## Load msg
    unless (open MSG, "$Conf{'queuemod'}/$list->{'name'}_$in{'id'}") {
	$param->{'error'}{'action'} = 'viewmod';
	&message('failed');
	&wwslog('info','do_viewmod: unable to read msg %s', $msg);
	return undef;
    }

    my $mail = new Mail::Internet [<MSG>];
    close MSG;

    foreach $field ('from','date','to','subject') {
	$param->{'msg'}{$field} =&MIME::Words::decode_mimewords($mail->head->get($field));
    }
    
    foreach $line (@{$mail->body()}) {
	$param->{'msg'}{'body'} .= $line; 
    }
    
    $param->{'id'} = $in{'id'};

    return 1;
}


sub do_viewmod {
    &wwslog('debug', 'do_viewmod(%s)', $in{'id'});
    my $msg;

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_viewmod: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_viewmod: no user');
	return undef;
    }
 
    unless ($in{'id'}) {
	$param->{'error'}{'argument'} = 'msgid';
	&message('missing_arg');
	&wwslog('info','do_viewmod: no msgid');
	return undef;
    }
   
    unless ($list->am_i('editor', $param->{'user'}{'email'})) {
	$param->{'error'}{'action'} = 'viewmod';
	&message('may_not');
	&wwslog('info','do_viewmod: %s not editor', $param->{'user'}{'email'});
	return undef;
    }

    unless (-d "$Conf{'queuemod'}/.$list->{'name'}_$in{'id'}") {
	unless (mkdir ("$Conf{'queuemod'}/.$list->{'name'}_$in{'id'}",0777)) {
	    &message('may_not_create_dir');
	    &wwslog('info','do_viewmod: unable to create %s', "$Conf{'queuemod'}/.$list->{'name'}_$in{'id'}");
	    return undef;
	}
	my $mhonarc_ressources ;
	if (-r "$Conf{'home'}/$list->{'name'}/mhonarc-ressources") {
	    $mhonarc_ressources =  "$Conf{'home'}/$list->{'name'}/mhonarc-ressources" ;
	}elsif (-r "$Conf{'etc'}/mhonarc-ressources"){
	    $mhonarc_ressources =  "$Conf{'etc'}/mhonarc-ressources" ;
	}elsif (-r "/var/lib/sympa/bin/etc/mhonarc-ressources"){
	    $mhonarc_ressources =  "/var/lib/sympa/bin/etc/mhonarc-ressources" ;
	}else {
	    do_log('notice',"Cannot find any MhOnArc ressource file");
	}

        do_log('notice',"xxxxx cd $Conf{'queuemod'}/.$list->{'name'}_$in{'id'} ; $wwsconf->{'mhonarc'}  -single -rcfile $mhonarc-ressources -definevars \"listname=$list->{'name'} hostname=$list->{'admin'}{'host'} \" $Conf{'queuemod'}/$list->{'name'}_$in{'id'} >msg00000.html");

	system "cd $Conf{'queuemod'}/.$list->{'name'}_$in{'id'} ; $wwsconf->{'mhonarc'}  -single -rcfile $mhonarc_ressources -definevars \"listname=$list->{'name'} hostname=$list->{'admin'}{'host'} \" $Conf{'queuemod'}/$list->{'name'}_$in{'id'} >msg00000.html";
    }

    if ($in{'file'}) {
	do_log('notice',"xxxxx attachement1 file $in{'file'} ");
	$in{'file'} =~ /\.(\w+)$/;
	$param->{'file_extension'} = $1;
	$param->{'file'} = "$Conf{'queuemod'}/.$list->{'name'}_$in{'id'}/$in{'file'}";
	$param->{'bypass'} = 1;
	do_log('notice',"xxxxx attachement2 param file $param->{'file'} ");
    }else {
	$param->{'file'} = "$Conf{'queuemod'}/.$list->{'name'}_$in{'id'}/msg00000.html" ;
	do_log('notice',"xxxxx param file $param->{'file'} ");
    }
    
    $param->{'base'} = sprintf "%s%s/viewmod/%s/%s/", $param->{'base_url'}, $param->{'path_cgi'}, $param->{'list'}, $in{'id'};
    $param->{'id'} = $in{'id'};

    return 1;
}


## Edition of list/sympa files
## No list -> sympa files (helpfile,...)
## TODO : upload
sub do_editfile {
    &wwslog('debug', 'do_editfile(%s)', $in{'file'});
    
    $param->{'subtitle'} = sprintf $param->{'subtitle'}, $in{'file'};

    unless ($in{'file'}) {
	$param->{'error'}{'argument'} = 'file';
	&message('missing_arg');
	&wwslog('info','do_editfile: no file');
	return undef;
    }

    unless (defined $filenames{$in{'file'}}) {
	$param->{'error'}{'file'} = $in{'file'};
	&message('file_not_editable');
	&wwslog('info','do_editfile: file %s not editable', $in{'file'});
	return undef;
    }

    $param->{'file'} = $in{'file'};

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_editfile: no user');
	return undef;
    }

    if ($param->{'list'}) {
	unless ($list->may_edit($in{'file'}, $param->{'user'}{'email'}) eq 'write') {
	    $param->{'error'}{'action'} = 'editfile';
	    &message('may_not');
	    &wwslog('info','do_editfile: not allowed');
	    return undef;
	}

	## Look for the template
	foreach my $dir ("$Conf{'home'}/$param->{'list'}","$Conf{'etc'}/templates","/var/lib/sympa/bin/etc/templates") {
	    if (-f "$dir/$in{'file'}") {
		$param->{'filepath'} = "$dir/$in{'file'}";
		last;
	    }
	}
    }else {
	unless (&List::is_listmaster($param->{'user'}{'email'})) {
	    $param->{'error'}{'argument'} = 'list';
	    &message('missing_arg');
	    &wwslog('info','do_editfile: no list');
	    return undef;
	}

	$param->{'filepath'} = "$Conf{'etc'}/templates/$in{'file'}";
    }

    if ($param->{'filepath'} && (! -r $param->{'filepath'})) {
	$param->{'error'}{'action'} = 'editfile';
	&message('failed');
	&wwslog('info','do_editfile: cannot read %s', $param->{'filepath'});
	return undef;
    }
    
    return 1;
}

## Saving of list files
sub do_savefile {
    &wwslog('debug', 'do_savefile(%s)', $in{'file'});
    
    $param->{'subtitle'} = sprintf $param->{'subtitle'}, $in{'file'};

    unless ($in{'file'}) {
	$param->{'error'}{'argument'} = 'file';
	&message('missing_arg');
	&wwslog('info','do_savefile: no file');
	return undef;
    }

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_savefile: no user');
	return undef;
    }

    if ($param->{'list'}) {
	unless ($list->am_i('owner', $param->{'user'}{'email'})) {
	    $param->{'error'}{'action'} = 'savefile';
	    &message('may_not');
	    &wwslog('info','do_savefile: not allowed');
	    return undef;
	}

	$param->{'filepath'} = "$Conf{'home'}/$list->{'name'}/$in{'file'}";
    }else {
	unless (&List::is_listmaster($param->{'user'}{'email'})) {
	    $param->{'error'}{'argument'} = 'list';
	    &message('missing_arg');
	    &wwslog('info','do_savefile: no list');
	    return undef;
	}

	$param->{'filepath'} = "$Conf{'etc'}/templates/$in{'file'}";
    }

    unless ((! -e $param->{'filepath'}) or (-w $param->{'filepath'})) {
	$param->{'error'}{'action'} = 'savefile';
	&message('failed');
	&wwslog('info','do_savefile: cannot write %s', $param->{'filepath'});
	return undef;
    }

    ## Keep the old file
    if (-e $param->{'filepath'}) {
	rename($param->{'filepath'}, "$param->{'filepath'}.orig");
    }

    if ($in{'content'} && ($in{'content'} !~ /^\s*$/)) {			
	## Save new file
	open FILE, ">$param->{'filepath'}";
	print FILE $in{'content'};
	close FILE;
    }else {
	&wwslog('info', 'do_savefile: deleting %s', $param->{'filepath'});
	unlink $param->{'filepath'};
    }

    $param->{'error'}{'action'} = 'savefile';
    &message('performed');

    return 1;
}

## Access to web archives
sub do_arc {
    &wwslog('debug', 'do_arc(%s, %s)', $in{'month'}, $in{'arc_file'});
    my $latest;
    my $index = $wwsconf->{'archive_default_index'};

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_arc: no list');
	return undef;
    }

    ## Access control

    unless (&List::request_action ('access_web_archive','md5',
				   {'listname' => $param->{'list'},
				    'sender' => $param->{'user'}{'email'},
				    'remote_host' => $param->{'remote_host'},
				    'remote_addr' => $param->{'remote_addr'}}) =~ /do_it/i) {
	$param->{'error'}{'action'} = 'arc';
	&message('may_not');
	&wwslog('info','do_arc: access denied for %s', $param->{'user'}{'email'});
	return undef;
   }

    
    ## Calendar
    unless (opendir ARC, "$wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}") {
	&message('no_archives');
	&wwslog('info','do_arc: no directory %s', "$wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}");
	return undef;
    }
    foreach $dir (sort grep(!/^\./,readdir ARC)) {
	if ($dir =~ /^(\d{4})-(\d{2})$/) {
	    $param->{'calendar'}{$1}{$2} = 1;
	    $latest = $dir;
	}
    }
    closedir ARC;

    ## Read html file
    $in{'month'} ||= $latest;
    
    unless ($in{'arc_file'}) {
	undef $latest;
	unless (opendir ARC, "$wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}/$in{'month'}") {
	    &wwslog('info',"unable to readdir $wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}/$in{'month'}");
	    &message('month_not_found');
	}
	foreach $file (grep(/^$index/,readdir ARC)) {
	    if ($file =~ /^$index(\d+)\.html$/) {
		$latest = $1 if ($latest < $1);
	    }
	}

	$in{'arc_file'} = $index.$latest.".html";
    }

    ## File exist ?
    unless (-r "$wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}/$in{'month'}/$in{'arc_file'}") {
	&wwslog('info',"unable to read $wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}/$in{'month'}/$in{'arc_file'}");
	&message('arc_not_found');
	return undef;
    }

    ## File type
    unless ($in{'arc_file'} =~ /^(mail\d+|msg\d+|thrd\d+)\.html$/) {
	$in{'arc_file'} =~ /\.(\w+)$/;
	$param->{'file_extension'} = $1;
	
	if ($param->{'file_extension'} !~ /^html$/i) {
	    $param->{'bypass'} = 1;
	}
    }

    $param->{'file'} = "$wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}/$in{'month'}/$in{'arc_file'}";
   

    $param->{'base'} = sprintf "%s%s/arc/%s/%s/", $param->{'base_url'}, $param->{'path_cgi'}, $param->{'list'}, $in{'month'};

    $param->{'archive_name'} = $in{'month'};


    return 1;
}

## Access to web archives
sub do_remove_arc {
    &wwslog('info', 'do_remove_arc : list %s, yyyy %s, mm %s, msgid %s', $in{'list'}, $in{'yyyy'}, $in{'month'}, $in{'msgid'});

    ## Access control should allow also email->sender to remove its messages
    #unless ( $param->{'is_owner'}) {
#	$param->{'error'}{'action'} = 'remove_arc';
#	&message('may_not_remove_arc');
#	&wwslog('info','remove_arc: access denied for %s', $param->{'user'}{'email'});
#	return undef;
#    }

    if ($in{'msgid'} =~ /NO-ID-FOUND\.mhonarc\.org/) {
	$param->{'error'}{'action'} = 'remove_arc';
	&message('may_not_remove_arc');
	&wwslog('info','remove_arc: no message id found');
	$param->{'status'} = 'no_msgid';
	return undef;
    } 
    ## 
    my $arcpath = "$wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}/$in{'yyyy'}-$in{'month'}";
    &wwslog('info','remove_arc: looking for %s in %s',$in{'msgid'},"$arcpath/arctxt");

    opendir ARC, "$arcpath/arctxt";
    my $message;
    $in{'msgid'} =~ s/\$/\\\$/g;
    foreach $file (grep (!/\./,readdir ARC)) {
	## &wwslog('info','remove_arc: scanning %s', $file);
	next unless (open MAIL,"$arcpath/arctxt/$file") ;
	while (<MAIL>) {
	    last if /^$/ ;
	    if ( /^Message-id:\s?<?$in{'msgid'}>?\s?/i ) {
		$message = $file ;
		last ;
	    }
	}
	close MAIL ;
	if ($message) {
	    unless (-d "$arcpath/deleted"){
		unless (mkdir ("$arcpath/deleted",0777)) {
		    &message('may_not_create_deleted_dir');
		    &wwslog('info',"remove_arc: unable to create $arcpath/deleted : $!");
		    $param->{'status'} = 'error';
		    last;
		}
	    }
	    unless (rename ("$arcpath/arctxt/$message","$arcpath/deleted/$message")) {
		&message('may_not_rename_deleted_message');
		&wwslog('info',"remove_arc: unable to rename message $arcpath/arctxt/$message");
		$param->{'status'} = 'error';
		last;
	    }
	    ## system "cd $arcpath ; $conf->{'mhonarc'} -rmm $in{'msgid'}";

  
	    my $file = "$Conf{'queueoutgoing'}/.remove.$list->{'name'}\@$list->{'admin'}{'host'}.$in{'yyyy'}-$in{'month'}.".time;

            unless (open REBUILD, ">$file") {
                $param->{'error'}{'action'} = 'remove';
                &message('failed');
	        &wwslog('info','do_remove: cannot create %s', $file);
	        return undef;
            }
 
            &do_log('info', 'create File: %s', $file);
    
            printf REBUILD ("%s\n",$in{'msgid'});
            close REBUILD;


	    &wwslog('info', 'do_remove_arc message marked for remove by archived %s', $message);
	    $param->{'status'} = 'done';

	    last;
	}else{
	    &wwslog('info', 'do_remove_arc : file %s do not match msgid', $file);
	}	    
    }
    unless ($message) {
	&wwslog('info', 'do_remove_arc : no file match msgid');
	$param->{'status'} = 'not_found';
    }

    closedir ARC;
    return 1;
}

## Output an initial form to search in web archives
sub do_arcsearch_form {
    &wwslog('debug', 'do_arcsearch_form(%s)', $param->{'list'});

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
        &message('missing_arg');
        &wwslog('info','do_arcsearch_form: no list');
        return undef;
    }

    ## Access control
    unless (&List::request_action ('access_web_archive','md5',
				   {'listname' => $param->{'list'},
				    'sender' => $param->{'user'}{'email'},
				    'remote_host' => $param->{'remote_host'},
				    'remote_addr' => $param->{'remote_addr'}}) =~ /do_it/i) {
	$param->{'error'}{'action'} = 'arcsearch_form';
        &message('may_not');
        &wwslog('info','do_arcsearch_form: access denied for %s', $param->{'user'}{'email'});
        return undef;
    }

    my $search_base = "$wwsconf->{'arc_path'}/$param->{'list'}\@$param->{'host'}";
    opendir ARC, "$search_base";
    foreach my $dir (sort {$b cmp $a} grep(!/^\./,readdir ARC)) {
        if ($dir =~ /^(\d{4})-(\d{2})$/) {
            push @{$param->{'yyyymm'}}, $dir;
        }
    }
    closedir ARC;

    $param->{'key_word'} = $in{'key_word'};
    $param->{'archive_name'} = $in{'archive_name'};
    
    return 1;
}

## Search in web archives
sub do_arcsearch {
    &wwslog('debug', 'do_arcsearch(%s)', $param->{'list'});

    unless ($param->{'list'}) {
        $param->{'error'}{'argument'} = 'list';
	&message('missing_argument');
        &wwslog('info','do_arcsearch: no list');
        return undef;
    }

    ## Access control
    unless (&List::request_action ('access_web_archive','md5',
				   {'listname' => $param->{'list'},
				    'sender' => $param->{'user'}{'email'},
				    'remote_host' => $param->{'remote_host'},
				    'remote_addr' => $param->{'remote_addr'}}) =~ /do_it/i) {
        $param->{'error'}{'action'} = 'arcsearch';
	&message('may_not');
        &wwslog('info','do_arcsearch: access denied for %s', $param->{'user'}{'email'});
        return undef;
    }

    use Marc::Search;

    my $search = new Marc::Search;
    $search->search_base ($wwsconf->{'arc_path'} . '/' . $param->{'list'} . '@' . $param->{'host'});
    $search->base_href ($param->{'base_url'}.$param->{'path_cgi'} . '/arc/' . $param->{'list'});
    
    $search->archive_name ($in{'archive_name'});
    
    if (defined($in{'directories'})) {
	$search->directories ($in{'directories'});
	foreach my $dir (split/\0/, $in{'directories'})	{
	    push @{$param->{'directories'}}, $dir;
	}
    }else {
	$search->directories ($search->archive_name);
	push @{$param->{'directories'}}, $search->archive_name;
    }
    
    if (defined $in{'previous'}) {
	$search->body_count ($in{'body_count'});
	$search->date_count ($in{'date_count'});
	$search->from_count ($in{'from_count'});
	$search->subj_count ($in{'subj_count'});
	$search->previous ($in{'previous'});
    }
    
    ## User didn't enter any search terms
    if ($in{'key_word'} =~ /^\s*$/) {
        $param->{'error'}{'argument'} = 'key_word';
	&message('missing_argument');
        &wwslog('info','do_arcsearch: no search term');
	return undef;
    }
    
    $param->{'key_word'} = $in{'key_word'};
    $in{'key_word'} =~ s/\@/\\\@/g;
    
    $search->limit ($in{'limit'});
    
    $search->age (1) 
	if (($in{'age'} eq 'new') or ($in{'age'} eq '1'));
    
    $search->match (1) 
	if (($in{'match'} eq 'partial') or ($in{'match'} eq '1'));
    
    my @words = split(/\s+/,$in{'key_word'});
    $search->words (\@words);
    $search->clean_words ($in{'key_word'});
    my @clean_words = @words;

    for my $i (0 .. $#words) {
        $words[$i] =~ s,/,\\/,g;
        $words[$i] = '\b' . $words[$i] . '\b' if ($in{'match'} eq 'exact');
    }
    $search->key_word (join('|',@words));
    
    if ($in{'case'} eq 'off') {
        $search->case(1);
        $search->key_word ('(?i)' . $search->key_word);
    }
    if ($in{'how'} eq 'any') {
        $search->function2 ($search->match_any(@words));
        $search->how ('any');
    }elsif ($in{'how'} eq 'all') {
        $search->function1 ($search->body_match_all(@clean_words,@words));
        $search->function2 ($search->match_all(@words));
        $search->how       ('all');
    }else {
        $search->function2 ($search->match_this(@words));
        $search->how       ('phrase');
    }

    $search->subj (defined($in{'subj'}));
    $search->from (defined($in{'from'}));
    $search->date (defined($in{'date'}));
    $search->body (defined($in{'body'}));
    
    $search->body (1) 
	if ( not ($search->subj)
	     and not ($search->from)
	     and not ($search->body)
	     and not ($search->date));

    my $searched = $search->search;
    
    if (defined($search->error)) {
	&wwslog('info','do_arcsearch_search_error : %s', $search->error);
    }
    
    $search->searched($searched);

    if ($searched < $search->file_count) {
	$param->{'continue'} = 1;
    }

    foreach my $field ('list','archive_name','age','body','case','date','from','how','limit','match','subj') {
	$param->{$field} = $in{$field};
    }

    $param->{'body_count'} = $search->body_count;
    $param->{'clean_words'} = $search->clean_words;
    $param->{'date_count'} = $search->date_count;
    $param->{'from_count'} = $search->from_count;
    $param->{'subj_count'} = $search->subj_count;
    
    $param->{'num'} = $search->file_count + 1;
    $param->{'searched'} = $search->searched;
    
    $param->{'res'} = $search->res;
    
    return 1;
}

# get pendings lists
sub do_get_pending_lists {

    &wwslog('debug', 'get_pending_lists');

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','get_pending_lists :  no user');
	return undef;
    }
    unless ( $param->{'is_listmaster'}) {
	&message('may_not');
	&do_log('info', 'Incorrect_privilege to get pending');
	return undef;
    } 

    foreach my $l ( &List::get_lists() ) {
	my $list = new List ($l);
	if ($list->{'admin'}{'status'} eq 'pending') {
	    $param->{'pending'}{$l}{'subject'} = $list->{'admin'}{'subject'};
	    $param->{'pending'}{$l}{'by'} = $list->{'admin'}{'creation'}{'email'};
	}
    }

    return 1;
}

## show a list parameters
sub do_set_pending_list_request {
    &wwslog('debug', 'set_pending_list(%s)',$in{'list'});
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','set_pending_list:  no user');
	return undef;
    }
    unless ( $param->{'is_listmaster'}) {
	&message('may_not');
	&do_log('info', 'Incorrect_privilege to open pending list %s from %s', $in{'list'},$param->{'user'}{'email'});
	return undef;
    } 

    $param->{'list_config'} = "$Conf{'home'}/$in{'list'}/config";
    $param->{'list_info'} = "$Conf{'home'}/$in{'list'}/info";
    $param->{'list_subject'} = $list->{'admin'}{'subject'};
    $param->{'list_request_by'} = $list->{'admin'}{'creation'}{'email'};
    $param->{'list_request_date'} = $list->{'admin'}{'creation'}{'date'};
    $param->{'list_serial'} = $list->{'admin'}{'serial'};
    $param->{'list_status'} = $list->{'admin'}{'status'};
 
    return 1;
}

## show a list parameters
sub do_install_pending_list {
    &wwslog('debug', 'do_install_pending_list(%s,%s,%s)',$in{'list'},$in{'status'},$in{'notify'});
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_install_pending_list:  no user');
	return undef;
    }
    unless ( $param->{'is_listmaster'}) {
	&message('Incorrect_privilege');
	&do_log('info', 'Incorrect_privilege to open pending list %s from %s', $in{'list'},$param->{'user'}{'email'});
	return undef;
    } 
        
    if ($list->{'admin'}{'status'} eq $in{'status'}) {
	&message('huummm_didnt_change_anything');
	&wwslog('info','view_pending_list: didn t change really the status, nothing to do');
	return undef ;
    }
    
    $param->{'list_config'} = "$Conf{'home'}/$in{'list'}/config";
    if ($in{'serial'} ne $list->{'admin'}{'serial'}) {
	&message('unable_to_write_config_some_one_else_is_editing_it');
	&wwslog('info','serial number as changed Sympa:%s Browser: %s',$in{'serial'},$list->{'admin'}{'serial'});
        return undef;
    }
    unless (open CONFIG, "$Conf{'home'}/$in{'list'}/config") {
	&message('unable_to_read_config');
	&wwslog('info','unable to read configuration file for list %s',$in{'list'});
	return undef;
    }
    unless (open CONFIGOLD, ">$Conf{'home'}/$in{'list'}/config.$in{'serial'}"){
	&message('unable_to_save_config');
	&wwslog('info','unable to write config.%s file for list %s',$in{'serial'},$in{'list'});
	return undef;
    }
    my $serial = $in{'serial'} + 1 ;
    
    unless (open CONFIGNEW, ">$Conf{'home'}/$in{'list'}/config.$serial"){
	&message('unable_to_save_config');
	&wwslog('info','unable to write config.%s file for list %s',$in{'serial'},$in{'list'});
	return undef;
    }
    
    while (<CONFIG>) {
	print CONFIGOLD $_ ;
	if (/^\s*status\s*(open|closed|pending)\s*$/i) {
	    print CONFIGNEW "status $in{'status'}\n";
	    $in{'status'} = 'setting' ;
	}elsif (/^\s*serial\s*(\d*)\s*$/i) {
	    print CONFIGNEW "serial $serial\n";
	    $in{'serial'} = 'setting' ;
	}else{
	    print CONFIGNEW $_ ;
	}
    }
    unless ($in{'status'}) {
	print CONFIGNEW "\nstatus $in{'status'}\n";
    }
    unless ($in{'serial'}) {
	print CONFIGNEW "\nstatus $in{'serial'}\n";
    }
    
    close CONFIG;
    close CONFIGOLD;
    close CONFIGNEW;
    &wwslog('info','new serial %d for list %s',$serial,$in{'list'});
    unless (rename ("$Conf{'home'}/$in{'list'}/config.$serial","$Conf{'home'}/$in{'list'}/config")){
	&message('unable_to_rename_config');
	&wwslog('info','unable to rename %s in %s (status %d)',"$Conf{'home'}/$in{'list'}/config.$serial","$Conf{'home'}/$in{'list'}/config,$!");
	return undef;
    }
    

    $param->{'aliases'}  = "#----------------- $in{'list'}\n";
    $param->{'aliases'} .= "$in{'list'}: \"| /var/lib/sympa/bin/queue $in{'list'}\"\n";
    $param->{'aliases'} .= "$in{'list'}-request: \"| /var/lib/sympa/bin/queue $in{'list'}-request\"\n";
    $param->{'aliases'} .= "$in{'list'}-owner: \"| /var/lib/sympa/bin/bouncequeue $in{'list'}\"\n";
    $param->{'aliases'} .= "$in{'list'}-unsubscribe: \"| /var/lib/sympa/bin/queue $in{'list'}-unsubscribe\"\n";
    $param->{'aliases'} .= "# $in{'list'}-subscribe: \"| /var/lib/sympa/bin/queue $in{'list'}-subscribe\"\n";

    return 1;
}
    

## create a liste using a list template. 
sub do_create_list {
    &wwslog('debug', 'do_create_list(%s,%s,%s)',$in{'listname'},$in{'subject'},$in{'template'});
    unless ($in{'listname'}) {
	&message('list_name_is_requiered');
	return undef;
    }
    $in{'listname'} = lc ($in{'listname'});

    unless ($in{'listname'} =~ /^[a-z0-9][a-z0-9\-\+\._]*$/i) {
	&message('incorrect_lisname');
	return undef;
    }
   
    unless ($in{'subject'}) {
	&message('subject_is_required');
	return undef;
    }
    unless ($in{'template'}) {
	&message('type_is_required');
	return undef;
    }
    unless ($in{'info'}) {
	&message('description_is_required');
	return undef;
    }

    unless ($in{'topics'}) {
	&message('topics_is_required');
	return undef;
    }

    if ( new List ($in{'listname'})) {
	&message('list_already_exists');
	&do_log('info', 'could not create already existing list %s for %s', $in{'listname'},$param->{'user'}{'email'});
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_create_list_request:  no user');
	return undef;
    }
    my $lang = $param->{'lang'};
    
    $param->{'create_action'} = &List::request_action('create_list','md5',
						      {'sender' => $param->{'user'}{'email'},
						      'remote_host' => $param->{'remote_host'},
						      'remote_addr' => $param->{'remote_addr'}});

    &wwslog('info',"do_create_list, get action : $param->{'create_action'} ");
    
    if ($param->{'create_action'} =~ /reject/) {
	$param->{'error'}{'action'} = 'create_list';
	&message('may_not');
	&wwslog('info','do_create_list: not allowed');
	return undef;
    }elsif ($param->{'create_action'} =~ /listmaster/i) {
	$param->{'status'} = 'pending' ;
    }elsif  ($param->{'create_action'} =~ /do_it/i) {
	$param->{'status'} = 'open' ;
    }else{
	$param->{'error'}{'action'} = 'create_list';
	&message('internal_scenario_error');
	&wwslog('info','do_create_list: internal error in scenario create_list');
	return undef;
    }
	     
    my $template_file ;
    if (-r "$Conf{'etc'}/create_list_templates/$in{'template'}/config.tpl") {
	$template_file = "$Conf{'etc'}/create_list_templates/$in{'template'}/config.tpl" ;
    }elsif(-r "/var/lib/sympa/bin/etc/create_list_templates/$in{'template'}/config.tpl") {
	$template_file = "/var/lib/sympa/bin/etc/create_list_templates/$in{'template'}/config.tpl";
    }else{
	&message('unable_to_open_template');
	&do_log('info', 'no template %s in %s NOR %s',$in{'template'},"$Conf{'etc'}/create_list_templates/$in{'template'}","/var/lib/sympa/bin/etc/create_list_templates/$in{'template'}");
	
	return undef;
    }
    
    unless (mkdir ("$Conf{'home'}/$in{'listname'}",0777)) {
	&message('unable_to_create_dir');
	&do_log('info', 'unable to create %s/%s : %s',$Conf{'home'},$in{'listname'},$?);
	return undef;
    }    
    
    my $parameters;
    $parameters->{'owner'}{'email'} = $param->{'user'}{'email'};
    $parameters->{'owner'}{'gecos'} = $param->{'user'}{'gcos'};
    $parameters->{'listname'} = $in{'listname'};
    $parameters->{'subject'} = $in{'subject'};
    $parameters->{'date'} = $param->{'date'};
    $parameters->{'date_epoch'} = time;
    $parameters->{'lang'} = $lang;
    $parameters->{'status'} = $param->{'status'};
    $parameters->{'topics'} = $in{'topics'};

    open CONFIG, ">$Conf{'home'}/$in{'listname'}/config";
    &parse_tpl($parameters, $template_file, CONFIG);
    close CONFIG;
    
    open INFO, ">$Conf{'home'}/$in{'listname'}/info" ;
    print INFO $in{'info'};
    close INFO;
    
    ## notify postmaster
    if ($param->{'create_action'} =~ /notify/) {
	&do_log('info','notify postmaster');
  	&List::send_notify_to_listmaster('request_list_creation',$in{'listname'},$parameters->{'owner'}{'email'});
    }
    return 1;
}

## Return the creation form
sub do_create_list_request {
    &wwslog('debug', 'do_create_list_request()');
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_create_list_request:  no user');
	return undef;
    }
    my $lang = $param->{'lang'};

    $param->{'create_action'} = &List::request_action('create_list','md5',
						  {'sender' => $param->{'user'}{'email'},
						   'remote_host' => $param->{'remote_host'},
						   'remote_addr' => $param->{'remote_addr'}});
    &wwslog('info',"do_create_list, get action : $param->{'create_action'} ");

    if ($param->{'create_action'} =~ /reject/) {
	$param->{'error'}{'action'} = 'create_list';
	&message('may_not');
	&wwslog('info','do_create_list: not allowed');
	return undef;
    }

    unless ($param->{'list_of_topics'} = &List::load_topics()) {
	&message('enable_to_load_list_of_topics');
    }

    unless ($param->{'list_list_tpl'} = &tools::get_list_list_tpl()) {
	&message('enable_to_load_create_list_templates');
    }	
    
    foreach my $template (keys %{$param->{'list_list_tpl'}}){
	$param->{'tpl_count'} ++ ;
    }
    return 1 ;

}

## WWSympa Home-Page
sub do_home {
    &wwslog('debug', 'do_home');

    my $topics = &List::load_topics();
    
    my $total = 0;
    foreach my $t (sort {$topics->{$a}{'order'} <=> $topics->{$b}{'order'}} keys %{$topics}) {
	my $current = $topics->{$t};
	$current->{'id'} = $t;
	$current->{'mod'} = $total % 3;
	push @{$param->{'topics'}}, $current;

	$total++;
    }
    
    push @{$param->{'topics'}}, {'id' => 'topicsless',
				 'mod' => $total};

    return 1;
}

sub do_editsubscriber {
    &wwslog('debug', 'do_editsubscriber(%s)', $in{'email'});

    my $user;

    unless ($param->{'is_owner'}) {
	$param->{'error'}{'action'} = 'editsubscriber';
	&message('may_not');
	&wwslog('info','do_editsubscriber: may not edit');
	return undef;
    }

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_editsubscriber: no list');
	return undef;
    }

    unless ($in{'email'}) {
	&message('no_email');
	&wwslog('info','do_editsubscriber: no email');
	return undef;
    }

    $in{'email'} = &tools::unescape_chars($in{'email'});

    unless($user = $list->get_subscriber($in{'email'})) {
	$param->{'error'}{'email'} = $in{'email'};
	&message('subscriber_not_found');
	&wwslog('info','do_editsubscriber: subscriber %s not found', $in{'email'});
	return undef;
    }

    $param->{'subscriber'} = $user;
    $param->{'subscriber'}{'escaped_email'} = &tools::escape_chars($param->{'subscriber'}{'email'});
    $param->{'subscriber'}{'date'} = &POSIX::strftime("%d %b %Y", localtime($user->{'date'}));

    ## Prefs
    $param->{'subscriber'}{'reception'} ||= 'mail';
    $param->{'subscriber'}{'visibility'} ||= 'noconceal';
    foreach my $m (keys %reception_mode) {		
	$param->{'reception'}{$m}{'description'} = $reception_mode{$m};
	if ($param->{'subscriber'}{'reception'} eq $m) {
	    $param->{'reception'}{$m}{'selected'} = 'SELECTED';
	}else {
	    $param->{'reception'}{$m}{'selected'} = '';
	}
    }

    ## Bounces
    if ($user->{'bounce'} =~ /^(\d+)\s+(\d+)\s+(\d+)(\s+(.*))?$/) {
	my @bounce = ($1, $2, $3, $5);
    	$param->{'subscriber'}{'first_bounce'} = &POSIX::strftime("%d %b %Y", localtime($bounce[0]));
    	$param->{'subscriber'}{'last_bounce'} = &POSIX::strftime("%d %b %Y", localtime($bounce[1]));
    	$param->{'subscriber'}{'bounce_count'} = $bounce[2];
	if ($bounce[3] =~ /^(\d+\.(\d+\.\d+))$/) {
	   $user->{'bounce_code'} = $1;
	   $user->{'bounce_status'} = $bounce_status{$2};
 	}	

	$param->{'previous_action'} = $in{'previous_action'};
    }

    return 1;
}

sub do_viewconfig {
    &wwslog('debug', 'do_viewconfig(%s)');
    unless ($param->{'list'}) {

	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_viewbounce: no list');
	return undef;
    }

    unless ($param->{'is_owner'}) {
	$param->{'error'}{'action'} = 'viewconfig';
	&message('may_not');
	&wwslog('info','do_viewconfig: may not view');
	return undef;
    }
    $param->{'list_config'} = "$Conf{'home'}/$in{'list'}/config";
    return 1;
}

sub do_viewbounce {
    &wwslog('debug', 'do_viewbounce(%s)', $in{'email'});

    unless ($param->{'is_owner'}) {
	$param->{'error'}{'action'} = 'viewbounce';
	&message('may_not');
	&wwslog('info','do_viewbounce: may not view');
	return undef;
    }

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_viewbounce: no list');
	return undef;
    }

    unless ($in{'email'}) {
	&message('no_email');
	&wwslog('info','do_viewbounce: no email');
	return undef;
    }

    my $escaped_email = &tools::escape_chars($in{'email'});

    $param->{'lastbounce_path'} = "$wwsconf->{'bounce_path'}/$param->{'list'}/$escaped_email";

    unless (-r $param->{'lastbounce_path'}) {
	$param->{'error'}{'email'} = $in{'email'};
	&message('no_bounce');
	&wwslog('info','do_viewbounce: no bounce %s', $param->{'lastbounce_path'});
	return undef;
    }

    return 1;
}

## some help for listmaster and developpers
sub do_scenario_test {
    &wwslog('debug', 'do_scenario_test');

    ## List available scenarii
    unless (opendir SCENARI, "/var/lib/sympa/bin/scenari/"){
	&wwslog('info',"do_scenario_test : unable to open /var/lib/sympa/bin/scenari");
	&message('scenari_wrong_access');
	return undef;
    }

    foreach $scfile (readdir SCENARI) {
	if ($scfile =~ /^(\w+)\.(\w+)/ ) {
	    $param->{'scenario'}{$1}{'defined'}=1 ;
	}
    }
    closedir SCENARI;
    foreach $l ( &List::get_lists() ) {
	$param->{'listname'}{$l}{'defined'}=1 ;
    }
    foreach $a ('smtp','md5','s/mime') {
	$param->{'auth_method'}{$a}{'define'}=1 ;
    }

    $param->{'scenario'}{$in{'scenario'}}{'selected'} = 'SELECTED' if $in{'scenario'};

    $param->{'listname'}{$in{'listname'}}{'selected'} = 'SELECTED' if $in{'listname'};
    
    $param->{'authmethod'}{$in{'auth_method'}}{'selected'} = 'SELECTED' if $in{'auth_method'};

    $param->{'email'} = $in{'email'};

    if ($in{'scenario'}) {
        my $operation = $in{'scenario'};
	&wwslog('debug', 'do_scenario_test: perform scenario_test');
	($param->{'scenario_condition'},$param->{'scenario_auth_method'},$param->{'scenario_action'}) = 
	    &List::request_action ($operation,$in{'auth_method'},
				   {'listname' => $in{'listname'},
				    'sender' => $in{'sender'},
				    'email' => $in{'email'},
				    'remote_host' => $in{'remote_host'},
				    'remote_addr' => $in{'remote_addr'}});	
    }
    return 1;
}

## Bouncing addresses review
sub do_reviewbouncing {
    &wwslog('debug', 'do_reviewbouncing(%d)', $in{'page'});
    my $size = $in{'size'} || $wwsconf->{'review_page_size'};

    unless ($in{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_reviewbouncing: no list');
	return undef;
    }

    unless ($param->{'is_owner'}) {
	$param->{'error'}{'action'} = 'reviewbouncing';
	&message('may_not');
	&wwslog('info','do_reviewbouncing: may not review');
	return undef;
    }

    unless ($param->{'total'} = $list->get_total_bouncing()) {
	&message('no_subscriber');
	&wwslog('info','do_reviewbouncing: no subscriber');
	return undef;
    }

    ## Owner
    $param->{'page'} = $in{'page'} || 1;
    $param->{'total_page'} = int ($param->{'total'} / $size);
    $param->{'total_page'} ++
	if ($param->{'total'} % $size);

    if ($param->{'page'} > $param->{'total_page'}) {
	$param->{'error'}{'page'} = $param->{'page'};
	&message('no_page');
	&wwslog('info','do_reviewbouncing: no page %d', $param->{'page'});
	return undef;
    }

    my @users;
    ## Members list
    for ($i = $list->get_first_bouncing_user(); $i; $i = $list->get_next_bouncing_user()) {
	$i->{'bounce'} =~ /^(\d+)\s+(\d+)\s+(\d+)(\s+(.*))?$/;
	$i->{'first_bounce'} = $1;
	$i->{'last_bounce'} = $2;
	$i->{'bounce_count'} = $3;
	if ($5 =~ /^(\d+)\.\d+\.\d+$/) {
	    $i->{'bounce_class'} = $1;
 	}	

	push @users, $i;
    }

    my $record;
    foreach my $i (sort 
		   {($b->{'last_bounce'} <=> $a->{'last_bounce'})
			|| ($b->{'bounce_class'} <=> $a->{'bounce_class'}) 
			|| ($b->{'bounce_count'} <=> $a->{'bounce_count'})}
		   @users) {
	$record++;

	if ($record > ( $size * ($param->{'page'} ) ) ) {
	    $param->{'next_page'} = $param->{'page'} + 1;
	    last;
	}

	next if ($record <= ( ($param->{'page'} - 1) *  $size));

	$i->{'first_bounce'} = &POSIX::strftime("%d %b %Y", localtime($i->{'first_bounce'}));
	$i->{'last_bounce'} = &POSIX::strftime("%d %b %Y", localtime($i->{'last_bounce'}));
	
	## Escape some weird chars
	$i->{'escaped_email'} = &tools::escape_chars($i->{'email'});

	push @{$param->{'members'}}, $i;
    }

#    ## Members list
#    for ($i = $list->get_first_bouncing_user(); $i; $i = $list->get_next_bouncing_user()) {
#	$record++;
#	
#	if ($record > ( $size * ($param->{'page'} ) ) ) {
#	    $param->{'next_page'} = $param->{'page'} + 1;
#	    last;
#	}
#	
#	next if ($record <= ( ($param->{'page'} - 1) *  $size));
#	
#	$i->{'bounce'} =~ /^(\d+)\s+(\d+)\s+(\d+)(\s+(.*))?$/;
#    	my @bounce = ($1, $2, $3, $5);
#    	$i->{'first_bounce'} = &POSIX::strftime("%d %b %Y", localtime($bounce[0]));
#    	$i->{'last_bounce'} = &POSIX::strftime("%d %b %Y", localtime($bounce[1]));
#    	$i->{'bounce_count'} = $bounce[2];
#	if ($bounce[3] =~ /^(\d+)\.\d+\.\d+$/) {
#	    $i->{'bounce_class'} = $1;
# 	}		
#
#	## Escape some weird chars
#	$i->{'escaped_email'} = &tools::escape_chars($i->{'email'});
#	
#	push @{$param->{'members'}}, $i;
#    }
    
    if ($param->{'page'} > 1) {
	$param->{'prev_page'} = $param->{'page'} - 1;
    }

    $param->{'size'} = $in{'size'};
    
    return 1;
}

sub do_resetbounce {
    &wwslog('debug', 'do_resetbounce(%s)', $in{'email'});

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_resetbounce: no list');
	return undef;
    }
    
    unless ($in{'email'}) {
	&message('no_email');
	&wwslog('info','do_resetbounce: no email');
	return undef;
    }

    $in{'email'} = &tools::unescape_chars($in{'email'});

    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_resetbounce: no user');
	return undef;
    }
 
    ## Require DEL privilege
    my $del_is = &List::request_action ('del','md5',
	{'listname' => $param->{'list'}, 
	 'sender' => $param->{'user'}{'email'},
	 'email' => $in{'email'},
	 'remote_host' => $param->{'remote_host'},
	 'remote_addr' => $param->{'remote_addr'}});
    
    unless ( $del_is =~ /do_it/) {
	$param->{'error'}{'action'} = 'del';
	&message('may_not');
	&wwslog('info','do_resetbounce: %s may not reset', $param->{'user'}{'email'});
	return undef;
    }

    my @emails = split /\0/, $in{'email'};

    foreach my $email (@emails) {
    
	unless ( $list->is_user($email) ) {
	    $param->{'error'}{'email'} = $email;
	    &message('not_subscribed');
	    &wwslog('info','do_del: %s not subscribed', $email);
	    return undef;
	}
	
	unless( $list->update_user($email, {'bounce' => 'NULL'})) {
	    $param->{'error'}{'action'} = 'del';
	    &message('failed');
	    &wwslog('info','do_resetbounce: failed update database for %s', $email);
	    return undef;
	}

	unless (unlink "$wwsconf->{'bounce_path'}/$param->{'list'}/$email") {
	    &wwslog('info','do_resetbounce: failed deleting %s', "$wwsconf->{'bounce_path'}/$param->{'list'}/$email");
	}
	
	&wwslog('info','do_resetbounce: bounces for %s reset ', $email);
	
    }

    return $in{'previous_action'} || 'review';
}

## Rebuild an archive using arctxt/
sub do_rebuildarc {
    &wwslog('debug', 'do_rebuildarc(%s, %s)', $param->{'list'}, $in{'month'});

    unless ($param->{'list'}) {
	$param->{'error'}{'argument'} = 'list';
	&message('missing_arg');
	&wwslog('info','do_rebuildarc: no list');
	return undef;
    }
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_rebuildarc: no user');
	return undef;
    }

    unless ($param->{'is_listmaster'}) {
	$param->{'error'}{'action'} = 'rebuildarc';
	&message('may_not');
	&wwslog('info','do_rebuildarc: not listmaster');
	return undef;
    }
  
    my $file = "$Conf{'queueoutgoing'}/.rebuild.$list->{'name'}\@$list->{'admin'}{'host'}";

    unless (open REBUILD, ">$file") {
	$param->{'error'}{'action'} = 'rebuildarc';
	&message('failed');
	&wwslog('info','do_rebuildarc: cannot create %s', $file);
	return undef;
    }
 
    &do_log('info', 'File: %s', $file);
    
    print REBUILD ' ';
    close REBUILD;

    $param->{'error'}{'action'} = 'rebuildarc';
    &message('performed');

    return 'admin';
}

## Rebuild all archives using arctxt/
sub do_rebuildallarc {
    &wwslog('debug', 'do_rebuildallarc');
    
    unless ($param->{'user'}{'email'}) {
	&message('no_user');
	&wwslog('info','do_rebuildallarc: no user');
	return undef;
    }

    unless ($param->{'is_listmaster'}) {
	$param->{'error'}{'action'} = 'rebuildallarc';
	&message('may_not');
	&wwslog('info','do_rebuildallarc: not listmaster');
	return undef;
    }
    foreach $l ( &List::get_lists() ) {
	my $list = new List ($l); 
	next if (undef ($list->{'admin'}{'web_archive'}{'defined'}));
        my $file = "$Conf{'queueoutgoing'}/.rebuild.$list->{'name'}\@$list->{'admin'}{'host'}";

	unless (open REBUILD, ">$file") {
	    $param->{'error'}{'action'} = 'rebuildarc';
	    &message('failed');
	    &wwslog('info','do_rebuildarc: cannot create %s', $file);
	    return undef;
	}
	
	&do_log('info', 'File: %s', $file);
    
	print REBUILD ' ';
	close REBUILD;
        
    }
    &message('performed');

    return 'admin';
}

## Search among lists
sub do_search_list {
    &wwslog('debug', 'do_search_list(%s)', $in{'filter'});

    unless ($in{'filter'}) {
	&message('no_filter');
	&wwslog('info','do_search_list: no filter');
	return undef;
    }
    
    ## Regexp
    $param->{'filter'} = $in{'filter'};
    $param->{'regexp'} = $param->{'filter'};
    $param->{'regexp'} =~ s/\\/\\\\/g;
    $param->{'regexp'} =~ s/\./\\\./g;
    $param->{'regexp'} =~ s/\*/\.\*/g;
    $param->{'regexp'} =~ s/\+/\\\+/g;
    $param->{'regexp'} =~ s/\?/\\\?/g;

    ## Members list
    my $record = 0;
    foreach $l ( &List::get_lists() ) {
	my $list = new List ($l);

	## Search filter
	next if (($list->{'name'} !~ /$param->{'regexp'}/i) 
		 && ($list->{'admin'}{'subject'} !~ /$param->{'regexp'}/i));
	
	my $action = &List::request_action ('visibility','md5',
					    {'listname' =>  $list->{'name'},
					     'sender' => $param->{'user'}{'email'}, 
					     'remote_host' => $param->{'remote_host'},
					     'remote_addr' => $param->{'remote_addr'}});
	
	next unless ($action eq 'do_it');

	$record++;
	$param->{'lists'}{$list->{'name'}} = {'host' => $list->{'admin'}{'host'},
					      'subject' => $list->{'admin'}{'subject'}};
    }

    $param->{'occurrence'} = $record;

    return 1;
}

sub do_edit_list {
    &wwslog('debug', 'do_edit_list()');

    foreach my $p (keys %in) {
	if ($p =~ /^config\.(\S+)$/) {
	    $param->{'config'}{$1} = $in{$p};
	}
    }

    return 1;
}

sub do_edit_list_request {
    &wwslog('debug', 'do_edit_list_request()');

#    unless ($param->{'user'}{'email'}) {
#	&message('no_user');
#	&wwslog('info','do_edit_list_request:  no user');
#	return undef;
#    }

#    unless (&List::request_action('edit_list', 'md5', 
#				  {'listname' =>  $list->{'name'},
#				   'sender' => $param->{'user'}{'email'},
#				   'remote_host' => $param->{'remote_host'},
#				   'remote_addr' => $param->{'remote_addr'}}) =~ /do_it/) {
#	$param->{'error'}{'action'} = 'edit_list_request';
#	&message('may_not');
#	&wwslog('info','do_edit_list: not allowed');
#	return undef;
#    }
    
    foreach my $pname (sort List::by_order keys %{$pinfo}) {

	my $p = &_prepare_data($pname, $pinfo{$pname}, $list->{'admin'}{$pname});

	$p->{'default'} = $list->{'admin'}{'defaults'}{$pname};

	push @{$param->{'param'}}, $p;	
    }

#    print "Content-type: text/plain\n\n";
#    &dump_var($list->{'admin'},0);
#    &dump_var($param->{'param'},0);

    return 1;
}

## Dump a variable's content
sub dump_var {
    my ($var, $level) = @_;

    if (ref($var)) {
	if (ref($var) eq 'ARRAY') {
	    foreach my $index (0..$#{$var}) {
		print STDOUT "\t"x$level.$index."\n";
		&dump_var($var->[$index], $level+1);
	    }
	}elsif (ref($var) eq 'HASH') {
	    foreach my $key (keys %{$var}) {
		print STDOUT "\t"x$level.$key."\n";
		&dump_var($var->{$key}, $level+1);
	    }    
	}
    }else {
	print STDOUT "\t"x$level.$var."\n";
    }
}

sub _prepare_data {
    my ($name, $struct, $data) = @_;
    &do_log('debug', '_prepare_data(%s, %s)', $name, $data);

    my $p_glob = {'name' => $name,
		  'title' => $struct->{'title'}{'fr'},
		  'description' => $struct->{'description'}{'fr'}
	      };

    ## Occurrences
    if ($struct->{'occurrence'} =~ /n$/) {
	$p_glob->{'occurrence'} = 'multiple';
	if (defined($data)) {
	    $data2 = $data;
	}else {
	    $data2 = [{}];
	}
    }else {
	$data2 = [$data];
    }
    
    my @all_p;

    foreach my $d (@{$data2}) {
	my $p = {};

	## Type of data
	if ($struct->{'scenario'}) {
	    $p->{'type'} = 'scenario';
	    my $list_of_scenario = $list->load_scenario_list($struct->{'scenario'});
	    
	    $list_of_scenario->{$d->{'name'}}{'selected'} = 1;
	    
	    foreach my $key (keys %{$list_of_scenario}) {
		$list_of_scenario->{$key}{'title'} = $list_of_scenario->{$key}{'title'}{'fr'};
	    }
	    
	    $p->{'value'} = $list_of_scenario;

	}elsif (ref ($struct->{'format'}) eq 'HASH') {
	    $p->{'type'} = 'paragraph';

	    foreach my $k (keys %{$struct->{'format'}}) {
		push @{$p->{'value'}}, &_prepare_data($k, $struct->{'format'}{$k}, $d->{$k});
	    }

	}elsif (ref ($struct->{'format'}) eq 'ARRAY') {
	    $p->{'type'} = 'enum';
	    foreach my $elt (@{$struct->{'format'}}) {
		$p->{'value'}{$elt} = 0;
	    }
	    $p->{'value'}{$d}{'selected'} = 1;

	}else {
	    $p->{'type'} = 'scalar';
	    $p->{'value'} = $d;

#	    if ($name eq 'days') {
#		foreach my $day (0..6) {
#		    $p->{$day}{'title'} = $week{'fr'}[$day];
#		}   
#		foreach my $day (@{$d}) {
#		    $p->{$day}{'selected'} = 1;
#		}
#		
#		$p->{'type'} = 'enum';
#	    }
	}

	push @all_p, $p;
    }

    if ($p_glob->{'occurrence'} eq 'multiple') {
	$p_glob->{'value'} = \@all_p;
    }else {
	foreach my $k (keys %{$all_p[0]}) {
	    $p_glob->{$k} = $all_p[0]->{$k};
	}
    }

    return $p_glob;
}

sub do_shared {
    &wwslog('debug', 'do_shared(%s)', $in{'path'});

    
    return 1;

}

sub do_upload_request {
    &wwslog('debug', 'do_upload_request()');

    return 1;
}

sub do_upload {
    &wwslog('debug', 'do_upload()');

    my $fh = $query->upload('uploaded_file');
    my $fn = $query->param('uploaded_file');

    $fn =~ /([^\/\\]+)$/;
    $fname = $1;
    $fname ||= 'uploaded';

    open FILE, ">/tmp/$fname";
 #   my $buffer;
 #   while (read($f, $buffer, 1024)) {
#	print FILE $buffer;
#    }

    while (<$fh>) {
	print FILE;
    }
    close FILE;

    return 1;
}




